import imp
import sys
from random import randint
from time import time


class Options(dict):
    """\
    Options dictionary. An instance of this is attached to each
    :class:`fresco.core.FrescoApp` instance, as a central store for
    configuration options.
    """

    def __getattr__(self, name):
        return self[name]

    def __setattr__(self, key, value):
        self[key] = value

    def update_from_file(self, path, load_all=False):
        """
        Update the instance with any symbols found in the python source file at
        `path`.

        :param path: The path to a python source file
        :param load_all: If true private symbols will also be loaded into the
                         options object.
        """
        module_name = 'fresco_options_%d%d' % (time(), randint(0, sys.maxsize))
        module = imp.load_source(module_name, path)
        self.update_from_object(module, load_all)
        del sys.modules[module_name]

    def update_from_object(self, ob, load_all=False):
        """
        Update the instance with any symbols listed in object `ob`
        :param load_all: If true private symbols will also be loaded into the
                         options object.
        """
        if load_all:
            symbols = dir(ob)
        else:
            symbols = getattr(ob, '__all__', None)
            if symbols is None:
                symbols = [name for name in dir(ob)
                                if not name.startswith('_')]
        self.update((s, getattr(ob, s)) for s in symbols)



from fresco import FrescoApp, GET, Response
from fresco.decorators import extract_getargs, json_response


class TestExtractGetArgs(object):

    def test_mixed_with_dispatcher_args(self):

        app = FrescoApp()

        @app.route(r'/<arg1:str>/<arg2:str>', GET)
        @extract_getargs(arg1=str, arg2=int)
        def handler(arg1, arg2):
            return Response([
                'Received %r:%s, %r:%s' % (arg1, type(arg1).__name__,
                                           arg2, type(arg2).__name__)
            ])

        with app.requestcontext('/foo/29'):
            assert app.view().content == ["Received 'foo':str, 29:int"]

    def test_query_args(self):

        @extract_getargs(arg1=str, arg2=int)
        def view(arg1, arg2):
            return Response([
                'Received %r:%s, %r:%s' % (arg1, type(arg1).__name__,
                                           arg2, type(arg2).__name__)
            ])

        app = FrescoApp()
        app.route('/', GET, view)
        with app.requestcontext('/?arg1=foo;arg2=29'):
            assert app.view().content == ["Received 'foo':str, 29:int"]

    def test_missing_args_with_one_default_argument(self):

        @extract_getargs(arg1=str, arg2=int)
        def view(arg1, arg2):
            return Response([
                'Received %r:%s, %r:%s' % (arg1, type(arg1).__name__,
                                           arg2, type(arg2).__name__)
            ])

        app = FrescoApp()
        app.route('/', GET, view)
        with app.requestcontext('/?arg1=foo'):
            assert app.view().status_code == 400

    def test_missing_args(self):

        @extract_getargs(arg1=str, arg2=int)
        def view(arg1, arg2=None):
            return Response([
                'Received %r:%s, %r:%s' % (arg1, type(arg1).__name__,
                                           arg2, type(arg2).__name__)
            ])

        app = FrescoApp()
        app.route('/', GET, view)
        with app.requestcontext('/?arg1=foo'):
            assert app.view().status == '200 OK'
            assert app.view().content == ["Received 'foo':str, None:NoneType"]

    def test_conversion_error_without_default(self):

        @extract_getargs(arg1=int)
        def view(arg1):
            return Response([])

        app = FrescoApp()
        app.route('/', GET, view)
        with app.requestcontext('/?arg1=foo'):
            assert app.view().status_code == 400

    def test_conversion_error_with_strict_checking(self):

        @extract_getargs(arg1=int, strict_checking=True)
        def view(arg1=None):
            return Response([
                'Received %r:%s' % (arg1, type(arg1).__name__)
            ])

        app = FrescoApp()
        app.route('/', GET, view)
        with app.requestcontext('/?arg1=foo'):
            assert app.view().status_code == 400

    def test_no_conversion_error_with_default(self):

        @extract_getargs(arg1=int)
        def view(arg1=None):
            return Response([
                'Received %r:%s' % (arg1, type(arg1).__name__)
            ])

        app = FrescoApp()
        app.route('/', GET, view)
        with app.requestcontext('/?arg1=foo'):
            assert app.view().content == ['Received None:NoneType']


class TestJSONResponse(object):

    def test_it_json_encodes(self):
        r = json_response({'l': [1, 2, 3]})
        assert r.content == ['{"l":[1,2,3]}']
        assert r.get_header('content-type') == 'application/json'

    def test_it_allows_custom_formatting(self):
        r = json_response({'l': [1, 2, 3]},
                          indent=1, separators=(', ', ': '))
        assert r.content == ['{\n "l": [\n  1, \n  2, \n  3\n ]\n}']

    def test_it_acts_as_a_decorator(self):
        @json_response()
        def f():
            return {'l': [1, 2, 3]}
        assert f().content == ['{"l":[1,2,3]}']



#
# Copyright (c) 2014 Oliver Cope. All rights reserved.
# See LICENSE.txt for terms of redistribution and use.

"""
Common utilities for writing applications in fresco
"""
from fresco.exceptions import NotFound

__all__ = ['object_or_404']


def object_or_404(ob, exception=NotFound):
    """
    Return the value of ``ob`` if it is not None. Otherwise raise a NotFound
    exception.
    """
    if ob is None:
        raise exception()
    return ob



# flake8: noqa

from sys import version_info

PY3 = version_info > (3, 0)
PY33 = version_info >= (3, 3)

if PY3:
    string_types = (str,)
    ustring_type = str
else:
    string_types = (basestring,)
    ustring_type = unicode

if PY33:
    from threading import get_ident

elif PY3:
    from _thread import get_ident

else:
    from thread import get_ident

if PY3:
    from urllib.parse import quote, quote_plus, unquote, urlparse, urlunparse, ParseResult

else:
    from urllib import quote, quote_plus, unquote
    from urlparse import urlparse, urlunparse, ParseResult

__all__ = 'PY3', 'string_types', 'ustring_type',\
          'get_ident', 'quote', 'quote_plus', 'unquote', 'urlparse', \
          'ParseResult'




__all__ = 'ResponseException', 'NotFound', 'Unauthorized',\
          'Forbidden', 'BadRequest', 'Redirect', 'RedirectTemporary',\
          'RedirectPermanent'

from fresco.response import Response


class ResponseException(Exception):
    """\
    An exception class with an associated :class:`fresco.response.Response`
    instance that will render a suitable error page.
    """
    response = None

    #: If true, the error response will be returned immediately without trying
    #: any other routes.
    is_final = True


class NotFound(ResponseException):
    """\
    Raised when a view needs to signal a 404 not found condition.
    """
    response = Response.not_found()

    def __init__(self, final=False):
        """
        ``final`` is false by default, meaning other routes will be offered the
        opportunity to handle the request. Only after other routes have been
        tried will an error response be returned.
        """
        self.is_final = final


class BadRequest(ResponseException):
    """\
    Return a 400 Bad Request response
    """
    response = Response(['Bad request'], status=400)


class Unauthorized(ResponseException):
    """\
    Return a 401 Unauthorized response.

    Use this when you want to flag that the user does not have permission to
    access a resource but may be able to gain access, eg by logging in.
    """
    response = Response(['Unauthorized'], status=401)


class Forbidden(ResponseException):
    """\
    Return a 403 Forbidden response

    Use this when you want to flag that the user does not have permission to
    access a resource and that there is no possiblity to do so (eg they are
    already logged in, but their account does not have the right access
    permissions)
    """
    response = Response(['Forbidden'], status=403)


class RedirectTemporary(ResponseException):
    """\
    Return a 302 Found response. Example:

        raise RedirectTemporary('http://example.org/')
    """
    def __init__(self, *args, **kwargs):
        self.response = Response.redirect(*args, **kwargs)

Redirect = RedirectTemporary


class RedirectPermanent(ResponseException):
    """\
    Return a 301 Moved Permanently response. Example:

        raise RedirectPermanent('http://example.org/')

    """
    def __init__(self, *args, **kwargs):
        self.response = Response.redirect_permanent(*args, **kwargs)


class RequestParseError(BadRequest):
    """\
    An error was encountered while parsing the HTTP request.
    """


class MethodNotAllowed(ResponseException):
    """\
    The URL is valid, but does not respond to the requested method
    """
    def __init__(self, valid_methods):
        super(MethodNotAllowed, self).__init__(valid_methods)
        self.response = Response.method_not_allowed(valid_methods)


class TooManyRequests(ResponseException):
    """
    Indicates that the user has sent too many
    requests in a given amount of time ("rate limiting").

    The response representations SHOULD include details explaining the
    condition, and MAY include a Retry-After header indicating how long
    to wait before making a new request.
    """
    response = Response(['Too many requests'], status=429)


class MissingRouteArg(BadRequest):
    """\
    A request argument was expected but not supplied.
    """



import pytest
from flea import Agent

from fresco import FrescoApp, GET, Response
from fresco.requestcontext import RequestContext


class TestRequestContext(object):

    def test_instantiation(self):
        """
        Can we cleanly instantiate a RequestContext object?
        """
        RequestContext()

    def test_app_populates_request_object(self):

        def view():
            from fresco.core import context
            assert context.request is not None
            return Response([''])

        app = FrescoApp()
        app.route('/', GET, view)

        assert Agent(app).get('/').body == ''

    def test_context_returns_correct_request_for_each_app(self):

        from time import sleep
        from threading import Thread, Lock, current_thread
        from functools import partial

        threadcount = 3
        itercount = 10
        output = []
        output_lock = Lock()

        def view():
            from fresco.core import context
            request_id = id(context.request)

            def generate_response():
                for i in range(itercount):
                    assert id(context.request) == request_id
                    output_lock.acquire()
                    output.append((request_id, current_thread(), i))
                    output_lock.release()
                    sleep(0.001)
                    yield str(request_id).encode('ascii')
            return Response(generate_response())

        app = FrescoApp()
        app.route('/', GET, view)

        threads = [Thread(target=partial(Agent(app).get, '/'))
                   for i in range(threadcount)]

        for t in threads:
            t.start()
        for t in threads:
            t.join()

        # Check that threaded requests were genuinely interleaved
        for i1, i2 in zip(output[:itercount - 1], output[1:itercount]):
            if i1[:2] != i2[:2]:
                break
        else:
            raise AssertionError("Output does not appear interleaved")
        assert len(output), threadcount * itercount

    def test_context_does_not_inherit_from_parent(self):
        c = RequestContext()
        c.push(foo=1)
        c.push(bar=2)
        with pytest.raises(AttributeError):
            c.foo

    def test_child_context_overrides_parent(self):
        c = RequestContext()
        c.push(foo=1)
        c.push(foo=2)
        assert c.foo == 2

    def test_pop_context_removes_keys(self):
        c = RequestContext()
        c.push(foo=1)
        c.push(foo=2)
        assert c.foo == 2
        c.pop()
        assert c.foo == 1



import sys
import os
import re
from calendar import timegm
from email.utils import formatdate, parsedate
from mock import Mock
from tempfile import mkstemp
from fresco import FrescoApp
from fresco.static import serve_static_file


class TestServeStaticFile():

    def setup(self):

        fh, tmpname = mkstemp()
        self.tmpname = tmpname
        self.mtime = int(os.path.getmtime(self.tmpname))
        os.close(fh)

    def teardown(self):
        os.unlink(self.tmpname)

    def parse_lmh(self, last_modified):
        return timegm(parsedate(last_modified))

    def make_ims(self, since, tz=0):
        return {'HTTP_IF_MODIFIED_SINCE': formatdate(since, tz)}

    def test_sets_last_modified_header(self):
        with FrescoApp().requestcontext():
            response = serve_static_file(self.tmpname)
            response.content.close()

        last_modified = response.get_header('Last-Modified')
        assert self.parse_lmh(last_modified) == self.mtime

    def test_full_response_when_if_modified_before_mtime(self):
        with FrescoApp().requestcontext(**self.make_ims(0)):
            response = serve_static_file(self.tmpname)
            response.content.close()

        last_modified = response.get_header('Last-Modified')
        assert response.status_code == 200
        assert self.parse_lmh(last_modified) == self.mtime

    def test_304_when_if_modified_after_mtime(self):
        with FrescoApp().requestcontext(**self.make_ims(self.mtime + 1)):
            response = serve_static_file(self.tmpname)
        assert response.status_code == 304

    def test_304_when_if_modified_after_mtime_with_tz1(self):
        with FrescoApp().requestcontext(**self.make_ims(self.mtime + 1, +1)):
            response = serve_static_file(self.tmpname)
        assert response.status_code == 304

    def test_304_when_if_modified_after_mtime_with_tz2(self):
        with FrescoApp().requestcontext(**self.make_ims(self.mtime + 1, -1)):
            response = serve_static_file(self.tmpname)
        assert response.status_code == 304

    def test_it_doesnt_fail_with_an_out_of_range_modified_since_value1(self):
        year = sys.maxsize
        with FrescoApp().requestcontext(
                HTTP_IF_MODIFIED_SINCE='"Wed, 09 Feb %s 04:46:40 GMT"' % year):
            response = serve_static_file(self.tmpname)
        assert response.status_code == 400

    def test_it_doesnt_fail_with_an_out_of_range_modified_since_value2(self):
        with FrescoApp().requestcontext(
                HTTP_IF_MODIFIED_SINCE='"Wed, 01 Feb -2001 04:46:40 GMT"'):
            response = serve_static_file(self.tmpname)
        assert response.status_code == 400

    def test_last_modified_format_is_correct(self):
        with FrescoApp().requestcontext():
            response = serve_static_file(self.tmpname)
            response.content.close()

        assert re.match(r'^\w{3}, \d{1,2} \w{3} \d{4} \d\d:\d\d:\d\d GMT',
                        response.get_header('last-modified'))

    def test_filewrapper_used_if_present(self):
        with FrescoApp().requestcontext() as c:
            file_wrapper = Mock(return_value=[])
            c.request.environ['wsgi.file_wrapper'] = file_wrapper
            serve_static_file(self.tmpname)

        file_wrapper.assert_called_once()

    def test_it_returns_not_found(self):
        with FrescoApp().requestcontext():
            response = serve_static_file(self.tmpname + 'x')
        assert response.status_code == 404

    def test_it_returns_forbidden(self):
        with FrescoApp().requestcontext():
            os.chmod(self.tmpname, 0)
            response = serve_static_file(self.tmpname)
        assert response.status_code == 403



__version__ = '0.3.13'

DEFAULT_CHARSET = 'UTF-8'

from .request import *  # noqa
from .requestcontext import context  # noqa
from .response import *  # noqa
from .core import *  # noqa
from .routing import *  # noqa
from .routeargs import *  # noqa

from .util.common import *  # noqa



# coding=UTF-8
from io import BytesIO

import pytest

from fresco import FrescoApp, exceptions
from fresco.tests import fixtures

context = FrescoApp().requestcontext


class TestRequestProperties(object):

    def test_url_script_name_only(self):
        with context(SCRIPT_NAME='/foo/bar', PATH_INFO='') as c:
            assert c.request.url == "http://localhost/foo/bar"

    def test_url_script_name_path_info(self):
        with context(SCRIPT_NAME='/foo/bar', PATH_INFO='/baz') as c:
            assert c.request.url == "http://localhost/foo/bar/baz"

    def test_url_normalizes_host_port(self):
        with context(HTTP_HOST='localhost:80') as c:
            assert c.request.url == "http://localhost/"
        with context(HTTP_HOST='localhost:81') as c:
            assert c.request.url == "http://localhost:81/"

    def test_url_normalizes_host_ssl_port(self):
        with context(environ={'wsgi.url_scheme': 'https'},
                     HTTP_HOST='localhost:443') as c:
            assert c.request.url == "https://localhost/"

    def test_url_ignores_server_port_if_host_header_present(self):
        with context(environ={'wsgi.url_scheme': 'https', 'SERVER_PORT': '81'},
                     HTTP_HOST='localhost') as c:
            assert c.request.url == "https://localhost/"

    def test_as_string(self):
        with context('http://example.org/foo?bar=baz') as c:
            assert str(c.request) == \
                        '<Request GET http://example.org/foo?bar=baz>'

    def test_url_returns_full_url(self):
        with context('http://example.org/foo?bar=baz') as c:
            assert c.request.url == 'http://example.org/foo?bar=baz'

    def test_path_returns_correct_path_when_script_name_empty(self):
        with context(SCRIPT_NAME='', PATH_INFO='/foo/bar') as c:
            assert c.request.path == '/foo/bar'

    def test_path_returns_correct_path(self):
        with context(SCRIPT_NAME='/foo', PATH_INFO='/bar') as c:
            assert c.request.path == '/foo/bar'

    def test_query_decodes_unicode(self):
        with context('/?q=%C3%A0') as c:
            assert c.request.query['q'] == b'\xc3\xa0'.decode('utf8')

    def test_form_decodes_unicode(self):
        with context('/?q=%C3%A0') as c:
            assert c.request.form['q'] == b'\xc3\xa0'.decode('utf8')

    def test_body_decodes_unicode(self):
        with context(wsgi_input=b'\xc3\xa0',
                     CONTENT_LENGTH='2',
                     CONTENT_TYPE='text/plain; charset=UTF-8') as c:
            assert c.request.body == b'\xc3\xa0'.decode('UTF-8')

    def test_body_raises_bad_request_for_invalid_encoding(self):
        with context(wsgi_input=b'\xc3a',
                     CONTENT_LENGTH='2',
                     CONTENT_TYPE='text/plain; charset=UTF-8') as c:
            with pytest.raises(exceptions.BadRequest):
                c.request.body

    def test_body_bytes_does_not_decode(self):
        with context(wsgi_input=b'\xc3a',
                     CONTENT_LENGTH='2',
                     CONTENT_TYPE='text/plain; charset=UTF-8') as c:
            c.request.body_bytes == b'\xc3a'

    def test_get_json_decodes_json(self):
        with context(wsgi_input=b'{"foo": "bar"}',
                     CONTENT_LENGTH='14',
                     CONTENT_TYPE='application/json') as c:
            c.request.get_json() == {'foo': 'bar'}

    def test_get_json_ignores_mime_type(self):
        with context(wsgi_input=b'{"foo": "bar"}',
                     CONTENT_LENGTH='14',
                     CONTENT_TYPE='application/broken') as c:
            c.request.get_json() == {'foo': 'bar'}

    def test_get_json_passes_args_to_decoder(self):
        with context(wsgi_input=b'{"foo": 1}',
                     CONTENT_LENGTH='10',
                     CONTENT_TYPE='application/broken') as c:
            c.request.get_json(parse_int=lambda s: s + '!') == {'foo': '1!'}


class TestPathEncoding(object):

    def test_url_is_quoted(self):

        with context(SCRIPT_NAME=fixtures.wsgi_unicode_path,
                     PATH_INFO='') as c:
            assert c.request.url == \
                    'http://localhost' + fixtures.quoted_unicode_path

    def test_application_url_is_quoted(self):

        with context(SCRIPT_NAME=fixtures.wsgi_unicode_path,
                     PATH_INFO='') as c:
            assert c.request.application_url == \
                    'http://localhost' + fixtures.quoted_unicode_path

    def test_parsed_url_is_quoted(self):
        with context(SCRIPT_NAME=fixtures.wsgi_unicode_path,
                     PATH_INFO='') as c:
            assert c.request.parsed_url.path == fixtures.quoted_unicode_path

    def test_path_is_unquoted(self):
        with context(SCRIPT_NAME=fixtures.wsgi_unicode_path,
                     PATH_INFO='') as c:
            assert c.request.path == fixtures.unquoted_unicode_path

    def test_script_name_is_unquoted(self):
        with context(SCRIPT_NAME=fixtures.wsgi_unicode_path,
                     PATH_INFO='') as c:
            assert c.request.script_name == fixtures.unquoted_unicode_path

    def test_path_info_is_unquoted(self):
        with context(PATH_INFO=fixtures.wsgi_unicode_path) as c:
            assert c.request.path_info == fixtures.unquoted_unicode_path

    def test_invalid_path_info_encoding_raises_bad_request(self):
        with context(PATH_INFO=fixtures.misquoted_wsgi_unicode_path) as c:
            with pytest.raises(exceptions.BadRequest):
                c.request.path_info

    def test_invalid_script_name_encoding_raises_bad_request(self):
        with context(SCRIPT_NAME=fixtures.misquoted_wsgi_unicode_path) as c:
            with pytest.raises(exceptions.BadRequest):
                c.request.script_name


class TestMakeURL(object):

    def assert_equal_with_query(self, url1, url2):
        if '?' not in url1:
            assert url1 == url2
            return

        base1, _, q1 = url1.partition('?')
        base2, _, q2 = url2.partition('?')
        assert base1 == base2
        assert sorted(q1.split(';')) == sorted(q2.split(';'))

    def test_it_returns_request_url(self):
        with context(SCRIPT_NAME='/script', PATH_INFO='/pathinfo') as c:
            assert c.request.make_url() == 'http://localhost/script/pathinfo'

    def test_it_doesnt_double_quote_request_url(self):
        with context(SCRIPT_NAME='/script name', PATH_INFO='/path info') as c:
            assert c.request.make_url() == \
                    'http://localhost/script%20name/path%20info'

    def test_it_doesnt_double_quote_supplied_path_info(self):
        with context() as c:
            assert c.request.make_url(PATH_INFO='/x y') == \
                    'http://localhost/x%20y'

    def test_can_replace_path(self):
        with context(SCRIPT_NAME='/script', PATH_INFO='/pathinfo') as c:
            assert c.request.make_url(path='/foo bar') == \
                    'http://localhost/foo%20bar'

    def test_it_joins_path(self):
        with context(SCRIPT_NAME='/script name', PATH_INFO='/path info/') as c:
            assert c.request.make_url(path='a/b c') == \
                    'http://localhost/script%20name/path%20info/a/b%20c'

    def test_query_not_included_by_default(self):
        with context(QUERY_STRING='query=foo') as c:
            assert c.request.make_url() == 'http://localhost/'

    def test_query_dict(self):
        with context() as c:
            self.assert_equal_with_query(
                c.request.make_url(query={'a': 1, 'b': '2 3'}),
                'http://localhost/?a=1;b=2+3')

    def test_query_kwargs(self):
        with context() as c:
            self.assert_equal_with_query(
                c.request.make_url(query={'a': 1}, b=2),
                'http://localhost/?a=1;b=2')

    def test_unicode_path(self):
        e = b'\xc3\xa9'.decode('utf8')  # e-acute
        with context() as c:
            assert c.request.make_url(path=e) == 'http://localhost/%C3%A9'

    def test_unicode_path_info(self):
        e = b'\xc3\xa9'.decode('utf8')  # e-acute
        with context() as c:
            assert c.request.make_url(PATH_INFO=e) == 'http://localhost/%C3%A9'

    def test_unicode_query(self):
        e = b'\xc3\xa9'.decode('utf8')  # e-acute
        with context() as c:
            assert c.request.make_url(query={'e': e}) == \
                        'http://localhost/?e=%C3%A9'

    def test_it_quotes_paths(self):
        with context() as c:
            assert c.request.make_url(path='/a b') == 'http://localhost/a%20b'

    def test_replace_query_replaces_existing(self):
        with context(QUERY_STRING='a=1') as c:
            assert c.request.make_url(query_replace={'a': '2'}) == \
                        'http://localhost/?a=2'

    def test_replace_query_adds_new_value(self):
        with context(QUERY_STRING='a=1') as c:
            self.assert_equal_with_query(
                c.request.make_url(query_replace={'b': '2'}),
                'http://localhost/?a=1;b=2')

    def test_add_query_doesnt_replace_existing(self):
        with context(QUERY_STRING='a=1') as c:
            self.assert_equal_with_query(
                c.request.make_url(query_add={'a': '2'}),
                'http://localhost/?a=1;a=2')

    def test_add_query_works_on_specified_query(self):
        with context(QUERY_STRING='a=1') as c:
            assert c.request.make_url(query='', query_add={'a': '2'}) == \
                        'http://localhost/?a=2'

    def test_add_query_does_not_mutate_request_query(self):
        with context(QUERY_STRING='a=1') as c:
            c.request.make_url(query_add={'a': '2'})
            assert list(c.request.query.allitems()) == [('a', '1')]


class TestResolveURL(object):

    def test_relative_path(self):
        with context(SCRIPT_NAME='/script', PATH_INFO='/pathinfo') as c:
            assert c.request.resolve_url('foo') == \
                        'http://localhost/script/foo'

    def test_absolute_path_unspecified_relative(self):
        with context(SCRIPT_NAME='/script', PATH_INFO='/pathinfo') as c:
            assert c.request.resolve_url('/foo', relative='app') == \
                        'http://localhost/script/foo'

    def test_absolute_path_app_relative(self):
        with context(SCRIPT_NAME='/script', PATH_INFO='/pathinfo') as c:
            assert c.request.resolve_url('/foo', relative='app') == \
                        'http://localhost/script/foo'

    def test_absolute_path_server_relative(self):
        with context(SCRIPT_NAME='/script', PATH_INFO='/pathinfo') as c:
            assert c.request.resolve_url('/foo', relative='server') == \
                        'http://localhost/foo'

    def test_ignores_server_port_if_host_header_present(self):
        with context(environ={'wsgi.url_scheme': 'https', 'SERVER_PORT': '81'},
                     HTTP_HOST='localhost') as c:
            assert c.request.resolve_url('/foo') == 'https://localhost/foo'


class TestCurrentRequest(object):

    def test_currentrequest_returns_current_request(self):
        from fresco import currentrequest
        with context() as c:
            assert currentrequest() is c.request

    def test_currentrequest_returns_None(self):
        from fresco import currentrequest
        assert currentrequest() is None


class TestFiles(object):

    filename = 'test.txt'
    filedata = '123456\n'
    boundary = '---------------------------1234'

    post_data = (
        '--{boundary}\r\n'
        'Content-Disposition: form-data; '
            'name="uploaded_file"; filename="{filename}"\r\n'
        'Content-Type: text/plain\r\n'
        '\r\n'
        '{filedata}\r\n'
        '--{boundary}'
        '--\r\n'
    ).format(boundary=boundary, filedata=filedata, filename=filename)\
     .encode('latin1')

    request_args = {'wsgi_input': post_data,
                    'REQUEST_METHOD': 'POST',
                    'CONTENT_TYPE': 'multipart/form-data; boundary=' +
                                        boundary,
                    'CONTENT_LENGTH': str(len(post_data))}

    def test_files_populated(self):

        with FrescoApp().requestcontext(**self.request_args) as c:
            request = c.request

        assert len(request.files) == 1
        assert 'uploaded_file' in request.files

    def test_file_content_available(self):

        with FrescoApp().requestcontext(**self.request_args) as c:
            request = c.request

        b = BytesIO()
        request.files['uploaded_file'].save(b)
        assert b.getvalue() == self.filedata.encode('latin1')

    def test_headers_available(self):

        with FrescoApp().requestcontext(**self.request_args) as c:
            request = c.request

        assert request.files['uploaded_file'].headers['content-type'] \
                == 'text/plain'
        assert request.files['uploaded_file'].filename == self.filename



from fresco.util.urls import normpath, make_query

# Greek letters as unicode strings (require multi-byte representation in UTF-8)
alpha = b'\xce\xb1'.decode('utf8')
beta = b'\xce\xb2'.decode('utf8')
gamma = b'\xce\xb3'.decode('utf8')


class TestNormPath(object):

    def test_empty_string(selfself):
        assert normpath('') == ''

    def test_root_url(self):
        assert normpath('/') == '/'

    def test_condenses_consecutive_slashes(self):
        assert normpath('//') == '/'
        assert normpath('///') == '/'

    def test_remove_single_dot(self):
        assert normpath('/./') == '/'

    def test_double_dot_interpreted(self):
        assert normpath('/../') == '/'
        assert normpath('/foo/../') == '/'

    def test_triple_dot_preserved(self):
        assert normpath('/.../') == '/.../'

    def test_combined_patterns(self):
        assert normpath('/..//../') == '/'
        assert normpath("/hello/.//dolly//") == "/hello/dolly/"
        assert normpath("///hello/.//dolly//./..//.//sailor") \
                == "/hello/sailor"

    def test_trailing_slash_preserved(self):
        assert normpath("/sliced/bread/") == "/sliced/bread/"


class TestMakeQuery(object):

    def test_make_query(self):
        assert sorted(make_query(a='1', b=2).split(';')) == ['a=1', 'b=2']
        assert make_query(a='one two three') == 'a=one+two+three'
        assert make_query(a=['one', 'two', 'three']) == 'a=one;a=two;a=three'

    def test_make_query_unicode(self):
        assert make_query(a=[alpha, beta, gamma], charset='utf8') \
                == 'a=%CE%B1;a=%CE%B2;a=%CE%B3'

    def test_make_query_unicode_default_encoding(self):
        assert make_query(a=[alpha, beta, gamma], charset='utf8') \
                == make_query(a=[alpha, beta, gamma])



import pytest

from fresco.util import security


class TestSecurity(object):

    def test_check_equal_constant_time_returns_equality(self):
        assert security.check_equal_constant_time('', '') is True
        assert security.check_equal_constant_time('abcabcabc', 'abcabcabc') \
                is True

    def test_check_equal_constant_time_returns_inequality(self):
        assert security.check_equal_constant_time(' ', '') is False
        assert security.check_equal_constant_time('abcabcabc', 'abcabcabx') \
                is False
        assert security.check_equal_constant_time('abcabcabc', 'abcabcabx') \
                is False
        assert security.check_equal_constant_time('abcabcabc', 'abcabcabde') \
                is False
        assert security.check_equal_constant_time('abcabcabc', 'abcabcab') \
                is False

    def test_check_equal_constant_time_raises_an_error_on_non_strings(self):
        with pytest.raises(TypeError):
            security.check_equal_constant_time(None, None)



from fresco import FrescoApp
from fresco.cookie import Cookie
from fresco.response import Response


class TestCannedResponses(object):

    def test_redirect(self):
        with FrescoApp().requestcontext():
            r = Response.redirect('/new_location')
        assert 'http://localhost/new_location' in ''.join(r.content)
        assert r.get_header('Location') == 'http://localhost/new_location'
        assert r.status == '302 Found'

    def test_redirect_temporary(self):
        with FrescoApp().requestcontext():
            r = Response.redirect_temporary('/new_location')
        assert 'http://localhost/new_location' in ''.join(r.content)
        assert r.get_header('Location') == 'http://localhost/new_location'
        assert r.status == '302 Found'

    def test_redirect_permanent(self):
        with FrescoApp().requestcontext():
            r = Response.redirect_permanent('/new_location')
        assert 'http://localhost/new_location' in ''.join(r.content)
        assert r.get_header('Location') == 'http://localhost/new_location'
        assert r.status == '301 Moved Permanently'

    def test_not_found(self):
        with FrescoApp().requestcontext():
            r = Response.not_found()
        assert 'not found' in ''.join(r.content).lower()
        assert r.status_code == 404

    def test_method_not_allowed(self):
        with FrescoApp().requestcontext():
            r = Response.method_not_allowed(valid_methods=('PUT', 'DELETE'))
        assert 'not allowed' in ''.join(r.content).lower()
        assert r.status == '405 Method Not Allowed'
        assert r.get_header('Allow') == 'PUT,DELETE'

    def test_meta_refresh(self):
        with FrescoApp().requestcontext():
            r = Response.meta_refresh('/next_page')
        content = ''.join(r.content)
        assert '<a href="http://localhost/next_page">' in content
        assert '<meta http-equiv="refresh" content="0;'\
               ' url=http://localhost/next_page">' in content
        assert r.status == '200 OK'
        assert r.get_header('Content-Type') == 'text/html'


class TestResponse(object):

    def test_unknown_response_header_from_python_symbol(self):
        r = Response(x_myheader='Foo', content_type=None)
        assert r.headers == [('X-Myheader', 'Foo')]

    def test_standard_headers_capitalized_from_symbol(self):
        r = Response(etag='foo')
        assert r.headers == [('ETag', 'foo')]

        r = Response(x_ua_compatible='foo')
        assert r.headers == [('X-UA-Compatible', 'foo')]

    def test_content_type_added_only_if_content(self):
        r = Response(content=[])
        assert 'Content-Type' in dict(r.headers)

        r = Response()
        assert 'Content-Type' not in dict(r.headers)

        r = Response(content=None)
        assert 'Content-Type' not in dict(r.headers)

    def test_add_response_header(self):
        r = Response(x_myheader='Foo')
        r = r.add_header('X-Myheader', 'Bar')
        assert r.headers == [('X-Myheader', 'Foo'), ('X-Myheader', 'Bar')]

    def test_set_content_type_header(self):
        r = Response(content_type='text/rtf; charset=ISO-8859-1')
        assert r.get_header('Content-Type') == 'text/rtf; charset=ISO-8859-1'

    def test_set_cookie(self):
        r = Response(set_cookie=Cookie(name='fruit', value='banana'))
        assert r.get_header('Set-Cookie') == 'fruit=banana;Version=1'

    def test_response_get_headers(self):

        r = Response(['whoa nelly!'],
                     content_type='text/plain',
                     x_test_header=["1", "2"])
        assert r.get_header('content-type') == 'text/plain'
        assert r.get_header('x-test-header') == '1,2'
        assert r.get_header('X-Test-Header') == '1,2'
        assert r.get_header('x-does-not-exist', 'boo!') == 'boo!'

    def test_response_buffered_sets_content_length(self):
        r = Response(['whoa nelly!']).buffered()
        assert r.get_header('content-length') == '11'



from fresco import FrescoApp, context, GET, Response
from fresco.middleware import XForwarded
from flea import Agent


class TestXForwarded(object):

    def get_app(self, *args, **kwargs):
        app = FrescoApp()

        @app.route('/', GET)
        def view():
            request = context.request
            return Response([request.url, ' ', request.environ['REMOTE_ADDR']])
        return XForwarded(app, *args, **kwargs)

    def test_forwards_x_forwarded_for(self):

        r = Agent(self.get_app())
        r = r.get('/', REMOTE_ADDR='127.0.0.1', HTTP_X_FORWARDED_FOR='1.2.3.4')
        url, addr = r.body.split(' ')
        assert addr == '1.2.3.4'

    def test_forwards_x_forwarded_host(self):

        r = Agent(self.get_app())
        r = r.get('/', REMOTE_ADDR='127.0.0.1',
                  HTTP_X_FORWARDED_HOST='frontendserver')
        url, addr = r.body.split(' ')
        assert url == 'http://frontendserver/'

    def test_forwards_x_forwarded_ssl(self):

        r = Agent(self.get_app())
        r = r.get('/', REMOTE_ADDR='127.0.0.1',
                  HTTP_X_FORWARDED_SSL='on',
                  HTTP_X_FORWARDED_HOST='localhost')
        url, addr = r.body.split(' ')
        assert url == 'https://localhost/', url

        r = Agent(self.get_app())
        r = r.get('/', REMOTE_ADDR='127.0.0.1',
                  HTTP_X_FORWARDED_SSL='off',
                  HTTP_X_FORWARDED_HOST='localhost')
        url, addr = r.body.split(' ')
        assert url == 'http://localhost/'

    def test_reports_first_trusted_ip(self):
        r = Agent(self.get_app(trusted=['127.0.0.1', '3.3.3.3']))
        r = r.get('/', REMOTE_ADDR='127.0.0.1',
                  HTTP_X_FORWARDED_FOR='1.1.1.1, 2.2.2.2, 3.3.3.3')
        url, addr = r.body.split(' ')
        assert addr == '2.2.2.2'

    def test_reports_first_listed_ip_as_remote_addr_when_untrusted(self):
        r = Agent(self.get_app())
        r = r.get('/', REMOTE_ADDR='127.0.0.1',
                  HTTP_X_FORWARDED_FOR='1.1.1.1, 2.2.2.2')
        url, addr = r.body.split(' ')
        assert addr == '1.1.1.1'



import pytest
from fresco.multidict import MultiDict


class CheckedMultiDict(MultiDict):

    def check_consistency(self):
        "Verify that the MultiDict has internal consistency"
        dictallitems = sorted((k, v)
                              for k, vs in self._dict.items()
                              for v in vs)
        orderallitems = sorted(self._order)
        assert list(sorted(dictallitems)) == list(sorted(orderallitems))

MultiDict = CheckedMultiDict


class TestMultDict(object):

    def test_getitem_returns_only_first(self):
        m = MultiDict([('a', 1), ('a', 2), ('b', 3)])
        assert m['a'] == 1

    def test_getitem_throws_keyerror(self):
        m = MultiDict([('a', 1), ('a', 2), ('b', 3)])
        with pytest.raises(KeyError):
            m['c']

    def test_get_returns_only_first(self):
        m = MultiDict([('a', 1), ('a', 2), ('b', 3)])
        assert m.get('a') == 1

    def test_get_returns_default(self):
        m = MultiDict()
        assert m.get('a') is None
        assert m.get('a', 'foo') == 'foo'

    def test_getlist_returns_list(self):
        m = MultiDict([('a', 1), ('a', 2)])
        assert m.getlist('a') == [1, 2]
        assert m.getlist('b') == []

    def test_copy_is_equal(self):
        m = MultiDict([('a', 1), ('a', 2)])
        assert list(m.allitems()) == list(m.copy().allitems())

    def test_copy_is_independent(self):
        m = MultiDict([('a', 1), ('a', 2)])
        n = m.copy()
        n['b'] = 'foo'
        assert list(m.allitems()) == [('a', 1), ('a', 2)]
        assert list(n.allitems()) == [('a', 1), ('a', 2), ('b', 'foo')]

    def test_fromkeys(self):
        m = MultiDict.fromkeys(['a', 'b'])
        assert list(m.allitems()) == [('a', None), ('b', None)]

    def test_fromkeys_with_value(self):
        m = MultiDict.fromkeys(['a', 'b'], 42)
        assert list(m.allitems()) == [('a', 42), ('b', 42)]

    def test_items_only_returns_first_of_each(self):
        m = MultiDict([('a', 1), ('a', 2)])
        assert list(m.items()) == [('a', 1)]

    def test_listitems_returns_lists(self):
        m = MultiDict([('a', 1), ('a', 2)])
        assert list(m.listitems()) == [('a', [1, 2])]

    def test_allitems_returns_all(self):
        m = MultiDict([('a', 1), ('a', 2)])
        assert list(m.allitems()) == [('a', 1), ('a', 2)]

    def test_allitems_returns_iterator(self):
        m = MultiDict([('a', 1), ('a', 2)])
        i = m.allitems()
        assert next(i) == ('a', 1)
        assert next(i) == ('a', 2)
        with pytest.raises(StopIteration):
            next(i)

    def test_keys(self):
        m = MultiDict([('a', 1), ('a', 2), ('b', 3)])
        i = m.keys()
        assert next(i) == 'a'
        assert next(i) == 'b'
        with pytest.raises(StopIteration):
            next(i)

    def test_values(self):
        m = MultiDict([('a', 1), ('a', 2), ('b', 3)])
        i = m.values()
        assert next(i) == 1
        assert next(i) == 3
        with pytest.raises(StopIteration):
            next(i)

    def test_listvalues(self):
        m = MultiDict([('a', 1), ('a', 2), ('b', 3)])
        assert list(m.listvalues()) == [[1, 2], [3]]

    def test_setitem(self):
        m = MultiDict()
        m['a'] = 1
        assert m['a'] == 1
        assert m.getlist('a') == [1]
        m.check_consistency()

    def test_setitem_replaces_existing(self):
        m = MultiDict()
        m['a'] = 1
        m['a'] = 2
        assert m.getlist('a') == [2]
        m.check_consistency()

    def test_delitem(self):
        m = MultiDict([('a', 1), ('a', 2), ('b', 3)])

        del m['a']
        assert list(m.allitems()) == [('b', 3)]
        m.check_consistency()

        m = MultiDict([('a', 1), ('a', 2), ('b', 3)])
        del m['b']
        assert list(m.allitems()) == [('a', 1), ('a', 2)]
        m.check_consistency()

    def test_iterable(self):
        m = MultiDict([('a', 1), ('a', 2), ('b', 3)])
        assert list(sorted(iter(m))) == ['a', 'b']

    def test_update_with_list(self):
        m = MultiDict([('a', 1), ('b', 2)])
        m.update([('a', 2), ('x', 2)])
        assert list(sorted(m.allitems())) == [('a', 2), ('b', 2), ('x', 2)]
        m.check_consistency()

    def test_update_with_dict(self):
        m = MultiDict([('a', 1), ('b', 2)])
        m.update({'a': 2, 'x': 2})
        assert list(sorted(m.allitems())) == [('a', 2), ('b', 2), ('x', 2)]
        m.check_consistency()

    def test_update_with_multidict(self):
        m = MultiDict([('a', 1), ('b', 2)])
        m.update(MultiDict([('a', 2), ('x', 2)]))
        assert list(sorted(m.allitems())) == [('a', 2), ('b', 2), ('x', 2)]
        m.check_consistency()

    def test_update_with_kwargs(self):
        m = MultiDict([('a', 1), ('b', 2)])
        m.update(a=2, x=2)
        assert sorted(list(m.allitems())) == [('a', 2), ('b', 2), ('x', 2)]
        m.check_consistency()

    def test_extend_with_list(self):
        m = MultiDict([('a', 1), ('b', 2)])
        m.extend([('a', 2), ('x', 2)])
        assert list(m.allitems()) == [('a', 1), ('b', 2), ('a', 2), ('x', 2)]
        m.check_consistency()

    def test_extend_with_dict(self):
        m = MultiDict([('a', 1), ('b', 2)])
        m.extend({'a': 2, 'x': 2})
        assert list(sorted(m.allitems())) == \
                [('a', 1), ('a', 2), ('b', 2), ('x', 2)]
        m.check_consistency()

    def test_extend_with_multidict(self):
        m = MultiDict([('a', 1), ('b', 2)])
        m.extend(MultiDict([('a', 2), ('x', 2)]))
        assert list(sorted(m.allitems())) == \
                [('a', 1), ('a', 2), ('b', 2), ('x', 2)]
        m.check_consistency()

    def test_extend_with_kwargs(self):
        m = MultiDict([('a', 1), ('b', 2)])
        m.extend(a=2, x=2)
        assert list(sorted(m.allitems())) == \
                [('a', 1), ('a', 2), ('b', 2), ('x', 2)]
        m.check_consistency()

    def test_pop(self):
        m = MultiDict([('a', 1), ('b', 2), ('a', 3)])
        assert m.pop('b') == 2
        assert list(sorted(m.allitems())) == [('a', 1), ('a', 3)]
        m.check_consistency()

    def test_pop_returns_first_item(self):
        m = MultiDict([('a', 1), ('b', 2), ('a', 3)])
        assert m.pop('a') == 1
        assert list(sorted(m.allitems())) == [('a', 3), ('b', 2)]
        m.check_consistency()

    def test_pop_with_default(self):
        m = MultiDict([])
        assert m.pop('c', 'foo') == 'foo'
        m.check_consistency()

    def test_popitem(self):
        m = MultiDict([('a', 1), ('b', 2), ('b', 3)])

        assert m.popitem() == ('b', 2)
        m.check_consistency()
        assert m.popitem() == ('b', 3)
        m.check_consistency()
        assert m.popitem() == ('a', 1)
        m.check_consistency()

        with pytest.raises(KeyError):
            m.popitem()

    def test_len(self):
        m = MultiDict([('a', 1), ('b', 2), ('b', 3)])
        assert len(m) == 2



from flea import Agent
from fresco import FrescoApp, GET, Response


class Receiver(object):

    def __init__(self):
        self.calls = []

    def __call__(self, *args, **kwargs):
        self.calls.append((args, kwargs))


class TestSignals(object):

    def test_route_matched_is_fired(self):

        def view():
            return Response([])

        app = FrescoApp()
        app.route('/0', GET, view)
        app.route('/1', GET, view)

        receiver = Receiver()
        app.route_matched.connect(receiver)

        Agent(app).get('/0')
        Agent(app).get('/1')

        assert len(receiver.calls) == 2

        args, kwargs = receiver.calls[0]
        assert args[0] is app
        assert kwargs.pop('route') is app.__routes__[0]
        assert kwargs.pop('view') is view
        assert kwargs.pop('request').path == '/0'
        assert kwargs == {}

        args, kwargs = receiver.calls[1]
        assert args[0] is app
        assert kwargs.pop('route') is app.__routes__[1]
        assert kwargs.pop('view') is view
        assert kwargs.pop('request').path == '/1'
        assert kwargs == {}

    def test_view_finished_is_fired(self):

        def view():
            return Response(['xyzzy'])

        app = FrescoApp()
        app.route('/', GET, view)

        receiver = Receiver()
        app.view_finished.connect(receiver)

        Agent(app).get('/')

        assert len(receiver.calls) == 1

        args, kwargs = receiver.calls[0]
        assert args[0] is app
        assert kwargs.pop('view') is view
        assert kwargs.pop('response').content == ['xyzzy']
        assert kwargs.pop('request') is not None
        assert kwargs == {}

    def test_signals_isolated(self):
        def view():
            return Response(['xyzzy'])

        app1 = FrescoApp()
        app1.route('/', GET, view)

        app2 = FrescoApp()
        app2.route('/', GET, view)

        receiver1 = Receiver()
        receiver2 = Receiver()
        app1.route_matched.connect(receiver1)
        app2.route_matched.connect(receiver2)

        Agent(app1).get('/')
        assert len(receiver1.calls) == 1
        assert len(receiver2.calls) == 0
        Agent(app2).get('/')
        assert len(receiver1.calls) == 1
        assert len(receiver2.calls) == 1



import sys
from tempfile import NamedTemporaryFile
from fresco.options import Options


class TestOptions(object):

    def test_options_dictionary_access(self):
        options = Options()
        options['x'] = 1
        assert options['x'] == 1

    def test_options_attribute_access(self):
        options = Options()
        options.x = 1
        assert options.x == 1

    def test_options_update_from_object(self):

        class Foo:
            a = 1
            b = 2

        options = Options()
        options.update_from_object(Foo())
        assert options == {'a': 1, 'b': 2}

    def test_options_update_from_object_loads_underscore_names(self):

        class Foo:
            pass

        options = Options()
        options.update_from_object(Foo(), True)
        assert '__module__' in options

    def test_options_update_from_file(self):

        with NamedTemporaryFile() as tmpfile:
            tmpfile.write(b"a = 1\nb = 2\n")
            tmpfile.flush()

            options = Options()
            options.update_from_file(tmpfile.name)
            assert options == {'a': 1, 'b': 2}

    def test_options_respects_all(self):
        with NamedTemporaryFile() as tmpfile:
            tmpfile.write(b"__all__ = ['a']\n"
                          b"a = 1\n"
                          b"b = 2\n")
            tmpfile.flush()

            options = Options()
            options.update_from_file(tmpfile.name)
            assert options == {'a': 1}

    def test_update_from_file_doesnt_add_module(self):
        with NamedTemporaryFile() as tmpfile:
            options = Options()
            saved_modules = list(sorted(sys.modules.keys()))
            options.update_from_file(tmpfile.name)
            assert list(sorted(sys.modules.keys())) == saved_modules



"""
Utilities for dealing with static file content
"""

import mimetypes
import os
from email.utils import formatdate, parsedate_tz, mktime_tz

from fresco import context
from fresco.response import Response, STATUS_OK, STATUS_NOT_MODIFIED
from fresco.util.wsgi import ClosingIterator

__all__ = 'serve_static_file',


def serve_static_file(path, default_charset="ISO-8859-1",
                      bufsize=8192):
    """
    Serve a static file located at ``path``.

    :returns: A :class:`~fresco.response.Response` object for the file at
              ``path``
    """

    try:
        mtime = os.path.getmtime(path)
    except OSError:
        return Response.not_found()

    request = context.request
    mod_since = request.get_header('if-modified-since')
    if mod_since is not None:
        try:
            mod_since = mktime_tz(parsedate_tz(mod_since))
        except (TypeError, OverflowError):
            return Response.bad_request()
        if int(mtime) <= int(mod_since):
            return Response(status=STATUS_NOT_MODIFIED)

    typ, enc = mimetypes.guess_type(path)
    if typ is None:
        typ = 'application/octet-stream'
    if typ.startswith('text/'):
        typ = typ + '; charset=%s' % default_charset

    if 'wsgi.file_wrapper' in request.environ:
        content_iterator = lambda f: \
                request.environ['wsgi.file_wrapper'](f, bufsize)
    else:
        content_iterator = lambda f: \
                ClosingIterator(iter(lambda: f.read(bufsize), ''), f.close)

    try:
        _file = open(path, 'rb')
    except IOError:
        return Response.forbidden()

    return Response(
        status=STATUS_OK,
        content_length=str(os.path.getsize(path)),
        last_modified=formatdate(mtime, localtime=False, usegmt=True),
        content_type=typ,
        content_encoding=enc,
        content=content_iterator(_file)
    )



"""
fresco.request
--------------

The :class:`Request` class models an incoming HTTP request, allowing access to
HTTP headers and request data (eg query string or submitted form data).
"""

import json
import posixpath
import re

from fresco import exceptions
from fresco.compat import (ustring_type, quote, urlparse, urlunparse,
                           ParseResult)
from fresco.util.wsgi import environ_to_unicode

import fresco

from .multidict import MultiDict
from .cookie import parse_cookie_header
from .util.urls import normpath, make_query

__all__ = 'Request', 'currentrequest'

KB = 1024
MB = 1024 * KB

# List of standard request header field names
# This is taken from the list available at
# http://en.wikipedia.org/wiki/List_of_HTTP_header_fields which combines
# RFC specified headers with commonly used non-standard header names
HEADER_NAMES = {
    'accept': 'Accept',
    'accept_charset': 'Accept-Charset',
    'accept_encoding': 'Accept-Encoding',
    'accept_language': 'Accept-Language',
    'accept_datetime': 'Accept-Datetime',
    'authorization': 'Authorization',
    'cache_control': 'Cache-Control',
    'connection': 'Connection',
    'cookie': 'Cookie',
    'content_length': 'Content-Length',
    'content_md5': 'Content-MD5',
    'content_type': 'Content-Type',
    'date': 'Date',
    'expect': 'Expect',
    'from': 'From',
    'host': 'Host',
    'if_match': 'If-Match',
    'if_modified_since': 'If-Modified-Since',
    'if_none_match': 'If-None-Match',
    'if_range': 'If-Range',
    'if_unmodified_since': 'If-Unmodified-Since',
    'max_forwards': 'Max-Forwards',
    'origin': 'Origin',
    'pragma': 'Pragma',
    'proxy_authorization': 'Proxy-Authorization',
    'range': 'Range',
    'referer': 'Referer',
    'te': 'TE',
    'user_agent': 'User-Agent',
    'upgrade': 'Upgrade',
    'via': 'Via',
    'warning': 'Warning',
    'common': 'Common',
    'field': 'Field',
    'x_requested_with': 'X-Requested-With',
    'dnt': 'DNT',
    'x_forwarded_for': 'X-Forwarded-For',
    'x_forwarded_host': 'X-Forwarded-Host',
    'x_forwarded_host': 'X-Forwarded-Host',
    'x_forwarded_proto': 'X-Forwarded-Proto',
    'front_end_https': 'Front-End-Https',
    'x_http_method_override': 'X-Http-Method-Override',
    'x_att_deviceid': 'X-ATT-DeviceId',
    'x_wap_profile': 'X-Wap-Profile',
    'proxy_connection': 'Proxy-Connection',
}


class Request(object):
    """
    Models an HTTP request, given a WSGI ``environ`` dictionary.
    """

    #: Maximum size for application/x-www-form-urlencoded post data, or maximum
    #: field size in multipart/form-data encoded data, not including file
    #: uploads
    MAX_SIZE = 16 * KB

    #: Maximum size for multipart/form-data encoded post data
    MAX_MULTIPART_SIZE = 2 * MB

    _body_bytes = None
    _parsed_content_type = None
    _form = None
    _files = None
    _query = None
    _cookies = None

    #: WSGI key for the session variable. The default value is configured for
    #: use with `beaker <http://beaker.groovie.org/>`_
    SESSION_ENV_KEY = 'beaker.session'

    #: The WSGI environ dict
    environ = None

    #: Encoding used to decode WSGI parameters, notably PATH_INFO and form data
    charset = fresco.DEFAULT_CHARSET

    #: The decoder class to use for JSON request payloads
    json_decoder_class = json.JSONDecoder

    def __new__(
        cls,
        environ,
        parse_content_type=re.compile(r'\s*(?:.*);\s*charset=([\w\d\-]+)\s*$')
    ):
        try:
            return environ['fresco.request']
        except KeyError:
            request = object.__new__(cls)
            request.environ = environ
            request.environ['fresco.request'] = request
            if 'CONTENT_TYPE' in environ:
                match = parse_content_type.match(environ['CONTENT_TYPE'])
                if match:
                    request.charset = match.group(1)
            return request

    def __str__(self):
        """
        Return a useful text representation of the request
        """
        return "<%s %s %s>" % (
                self.__class__.__name__,
                self.method,
                self.url,
        )

    @property
    def form(self):
        """
        Return the contents of submitted form data

        This will return the ``POST`` or ``PUT`` data when available, otherwise
        querystring (``GET``)  data. Querystring data is always available via
        the ``query`` property.
        """
        if self._form is None:
            if self.environ['REQUEST_METHOD'] in ('PUT', 'POST'):
                self._form = MultiDict(
                    parse_post(self.environ,
                               self.environ['wsgi.input'],
                               self.charset,
                               self.MAX_SIZE,
                               self.MAX_MULTIPART_SIZE)
                )
            else:
                query = environ_to_unicode(
                            self.environ.get('QUERY_STRING', ''))
                self._form = MultiDict(parse_querystring(query, self.charset))
        return self._form

    @property
    def content_type_encoding(self):
        if self._parsed_content_type is None:
            self._parsed_content_type = get_content_type_info(self.environ)
        return self._parsed_content_type.encoding

    @property
    def content_type(self):
        if self._parsed_content_type is None:
            self._parsed_content_type = get_content_type_info(self.environ)
        return self._parsed_content_type.content_type

    @property
    def body_bytes(self):
        if self._body_bytes is None:
            self._body_bytes = get_body_bytes(self.environ, self.MAX_SIZE)
        return self._body_bytes

    @property
    def body(self):
        body = self.body_bytes
        if body is None:
            return None
        encoding = self.content_type_encoding or self.charset
        try:
            return body.decode(encoding)
        except UnicodeDecodeError:
            raise exceptions.RequestParseError(
                "Payload contains data that could not be decoded using " +
                encoding)

    def get_json(self, *args, **kwargs):
        """
        Return a decoded JSON request body.

        ``*args`` and ``**kwargs`` are passed to the ``JSONDecoder``
        constructor.

        This will try to decode a json string regardless of the mime type sent
        by the client.
        """
        return self.json_decoder_class(*args, **kwargs).decode(self.body)

    @property
    def files(self):
        """
        Return a MultiDict of all ``FileUpload`` objects available.
        """
        if self._files is None:
            self._files = MultiDict((k, v) for k, v in self.form.allitems()
                                           if isinstance(v, FileUpload))
        return self._files

    @property
    def query(self):
        """
        Return a ``MultiDict`` of any querystring submitted data.

        This is available regardless of whether the original request was a
        ``GET`` request.

        Synopsis::

            >>> from fresco import FrescoApp
            >>> with FrescoApp().requestcontext('/?animal=moose') as c:
            ...     c.request.query['animal']
            u'moose'

        This property always returns querystring data, regardless of the
        request method used.
        """
        if self._query is None:
            query = environ_to_unicode(self.environ.get('QUERY_STRING', ''))
            self._query = MultiDict(parse_querystring(query, self.charset))

        return self._query

    def __getitem__(self, key):
        """
        Return the value of ``key`` from submitted form values.
        """
        marker = []
        v = self.get(key, marker)
        if v is marker:
            raise KeyError(key)
        return v

    def get(self, key, default=None):
        """
        Look up ``key`` in submitted form values
        """
        return self.form.get(key, default)

    def getlist(self, key):
        """
        Return a list of submitted form values for ``key``
        """
        return self.form.getlist(key)

    def __contains__(self, key):
        """
        Return ``True`` if ``key`` is in the submitted form values
        """
        return key in self.form

    @property
    def cookies(self):
        """
        Return a :class:`fresco.multidict.MultiDict` of cookies read from the
        request headers::

            >>> from fresco import FrescoApp
            >>> with FrescoApp().requestcontext(
            ...     HTTP_COOKIE='''$Version="1";
            ...     Customer="WILE_E_COYOTE";
            ...     Part="Rocket_0001";
            ...     Part="Catapult_0032"
            ... ''') as c:
            ...     request = c.request
            ...
            >>> [item.value for item in request.cookies.getlist('Customer')]
            ['WILE_E_COYOTE']
            >>> [item.value for item in request.cookies.getlist('Part')]
            ['Rocket_0001', 'Catapult_0032']

        See rfc2109, section 4.4
        """
        if self._cookies is None:
            self._cookies = MultiDict(
                (cookie.name, cookie)
                for cookie in parse_cookie_header(self.get_header("Cookie"))
            )
        return self._cookies

    def get_header(self, name, default=None):
        """
        Return an arbitrary HTTP header from the request.

        :param name: HTTP header name, eg 'User-Agent' or 'If-Modified-Since'.
        :param default: default value to return if the header is not set.

        Headers in the original HTTP request are always formatted like this::

            If-Modified-Since: Thu, 04 Jan 2007 21:41:08 GMT

        However in the WSGI environment they appear as follows::

            {'HTTP_IF_MODIFIED_SINCE': 'Thu, 04 Jan 2007 21:41:08 GMT'}

        This method expects the former style (eg ``If-Modified-Since``) and is
        not case sensitive.
        """
        return self.environ.get(
            'HTTP_' + name.upper().replace('-', '_'),
            default
        )

    @property
    def path(self):
        """\
        Return the path component of the requested URL
        """
        return environ_to_unicode(self.environ.get('SCRIPT_NAME', '') +
                                  self.environ.get('PATH_INFO', ''))

    @property
    def url(self):
        """\
        Return the full URL, including query parameters.
        """
        return urlunparse(self.parsed_url)

    @property
    def application_url(self):
        """\
        Return the base URL of the WSGI application (up to SCRIPT_NAME but not
        including PATH_INFO or query information).

        Synopsis::

            >>> from fresco import FrescoApp
            >>> with FrescoApp().requestcontext(HTTP_HOST='example.com',
            ...                                 SCRIPT_NAME='/animals',
            ...                                 PATH_INFO='/lion.html') as c:
            ...     c.request.application_url
            u'http://example.com/animals'
        """
        scheme, netloc, path, params, query, frag = self.parsed_url

        return urlunparse((scheme, netloc,
                           quote(self.script_name.encode(self.charset)), '',
                           '', ''))

    @property
    def parsed_url(self):
        """\
        Return the current URL as a tuple of the form::

            (addressing scheme, network location, path,
             parameters, query, fragment identifier)

        Synopsis::

            >>> from fresco import FrescoApp
            >>> app = FrescoApp()
            >>> with app.requestcontext(
            ...          'https://example.com/animals/view?name=lion') as c:
            ...     c.request.parsed_url  # doctest: +ELLIPSIS
            ParseResult(scheme=u'https', netloc=u'example.com', ...)

        Components are returned as unicode strings
        """
        env = self.environ.get
        script_name = quote(self.script_name.encode(self.charset))
        path_info = quote(self.path_info.encode(self.charset))
        query_string = self.query_string
        scheme = environ_to_unicode(env('wsgi.url_scheme', 'http'))

        try:
            host = environ_to_unicode(self.environ['HTTP_HOST'])
            if ':' in host:
                host, port = host.split(':', 1)
            else:
                if scheme == 'https':
                    port = '443'
                else:
                    port = '80'
        except KeyError:
            host = environ_to_unicode(self.environ['SERVER_NAME'])
            port = environ_to_unicode(self.environ['SERVER_PORT'])

        if (scheme == 'http' and port == '80') \
                or (scheme == 'https' and port == '443'):
            netloc = host
        else:
            netloc = host + ':' + port

        return ParseResult(
            scheme,
            netloc,
            script_name + path_info,
            '',  # Params
            query_string,
            '',  # Fragment
        )

    @property
    def path_info(self):
        """
        The PATH_INFO value as a unicode string

        Note that PATH_INFO is already unquoted by the server
        """
        try:
            return environ_to_unicode(self.environ.get('PATH_INFO', ''),
                                      self.charset)
        except UnicodeDecodeError:
            raise exceptions.BadRequest

    @property
    def script_name(self):
        """\
        The SCRIPT_NAME value as a unicode string

        Note that SCRIPT_NAME is already unquoted by the server
        """
        try:
            return environ_to_unicode(self.environ.get('SCRIPT_NAME', ''),
                                      self.charset)
        except UnicodeDecodeError:
            raise exceptions.BadRequest

    @property
    def query_string(self):
        """\
        The QUERY_STRING value as a unicode string
        """
        try:
            return environ_to_unicode(self.environ.get("QUERY_STRING"))
        except AttributeError:
            return None

    @property
    def referrer(self):
        """
        Return the HTTP referer header, or ``None`` if this is not available.
        """
        try:
            return environ_to_unicode(self.environ.get("HTTP_REFERER"))
        except AttributeError:
            return None

    @property
    def method(self):
        """
        Return the HTTP method used for the request, eg ``GET`` or ``POST``.
        """
        return environ_to_unicode(self.environ["REQUEST_METHOD"])

    @property
    def remote_addr(self):
        """
        Return the remote address of the client
        """
        try:
            return environ_to_unicode(self.environ.get("REMOTE_ADDR"))
        except AttributeError:
            return None

    @property
    def session(self):
        """
        Return the session associated with this request.

        Requires a session object to have been inserted into the WSGI
        environment by a middleware application.
        """
        return self.environ.get(self.SESSION_ENV_KEY)

    def make_url(self, scheme=None, netloc=None, path=None, parameters=None,
                 query=None, query_add=None, query_replace=None, fragment=None,
                 SCRIPT_NAME=None, PATH_INFO=None, **kwargs):
        r"""\
        Make a new URL based on the current URL, replacing any of the six
        URL elements (scheme, netloc, path, parameters, query or fragment). The
        current request's query string is not included in the generated URL
        unless you explicitly pass it in.

        :param scheme:      The URL scheme, eg ``http`` or ``https``
        :param netloc:      The netloc portion of the URL, eg 'example.com:80'
        :param path:        The path portion of the URL, eg '/my/page.html'
        :param parameters:  RFC2396 path parameters (see also the stdlib
                            urlparse module)
        :param fragment:    The URL fragment
        :param query:       Query data, as a list of tuples, a dict or a string
        :param query_add:   Query parameters to add. If the current url already
                            contains query parameters with the same name, these
                            will be supplemented with the new values.
        :param query_replace: Query parameters to replace. If the current url
                              already contains query parameters with the same
                              name, these will be replaced with the new values.
        :param PATH_INFO:   The PATH_INFO portion of the path. Overrides
                            ``path``
        :param SCRIPT_NAME: The SCRIPT_NAME portion of the path. Overrides
                            ``path``
        :param kwargs:      Any remaining keyword arguments are appended to the
                            querystring
        :rtype: str

        All arguments (other than query related arguments, see examples below)
        may be byte strings or unicode strings (native strings in python 3).
        Where necessary values will be converted to utf-8 encoded byte strings.

        Examples:

        Reproduce the request URL::

            >>> from fresco import FrescoApp
            >>> with FrescoApp().requestcontext(HTTP_HOST='example.com',
            ...                                 SCRIPT_NAME='/fruitsalad',
            ...                                 PATH_INFO='/banana',
            ...                                 QUERY_STRING='cream=n') as c:
            ...     request = c.request

            >>> request.make_url(query=request.query)
            u'http://example.com/fruitsalad/banana?cream=n'

        Replace the URL scheme::

            >>> request.make_url(scheme='ftp')
            u'ftp://example.com/fruitsalad/banana'

        Replace the entire path::

            >>> request.make_url(path='/sausages')
            u'http://example.com/sausages'

        Replace just the path_info portion::

            >>> request.make_url(PATH_INFO='/sausages')
            u'http://example.com/fruitsalad/sausages'

        Add a query string::

            request.make_url(query={'portions': '2'}, cherries='n')
            u'http://example.com/fruitsalad/banana?portions=2;cherries='n'

        If a relative path is passed, the returned URL is joined to the old in
        the same way as a web browser would interpret a relative HREF in a
        document at the current location::

            >>> request.make_url(path='kiwi')
            u'http://example.com/fruitsalad/kiwi'

            >>> request.make_url(path='../strawberry')
            u'http://example.com/strawberry'

            >>> request.make_url(path='../../../plum')
            u'http://example.com/plum'

        The ``query`` argument can take a dictionary, a list of ``(name,
        value)`` pairs or any other type convertable to a MultiDict::

            >>> request.make_url(query='a=tokyo&b=milan')
            u'http://example.com/fruitsalad/banana?a=tokyo&b=milan'

            >>> request.make_url(query={'a': 'tokyo', 'b': 'milan'})
            u'http://example.com/fruitsalad/banana?a=tokyo;b=milan'

            >>> request.make_url(query=[('a', 'tokyo'),
            ...                         ('b', 'milan'),
            ...                         ('b', 'paris')])
            u'http://example.com/fruitsalad/banana?a=tokyo;b=milan;b=paris'

        The ``query_add`` and ``query_replace`` arguments add or replace
        values in an existing query string. If either of these is specified
        then the current request's query string (or value of ``query``, if
        specified) will be extended with the given values.
        These arguments take a dict, a list of ``(key, value)`` tuples, or any
        other type convertable to a MultiDict.
        """
        url = []
        parsed_url = self.parsed_url

        if path is not None:
            if not isinstance(path, bytes):
                path = path.encode('utf-8')
            path = quote(path)

            if path[0] != '/':
                path = posixpath.join(
                        posixpath.dirname(parsed_url[2]), path)
                path = posixpath.normpath(path)

        elif SCRIPT_NAME is not None or PATH_INFO is not None:
            if SCRIPT_NAME is None:
                SCRIPT_NAME = self.script_name
            if PATH_INFO is None:
                PATH_INFO = self.path_info
            path = SCRIPT_NAME + PATH_INFO
            if not isinstance(path, bytes):
                path = path.encode('utf-8')
            path = quote(path)

        else:
            path = parsed_url[2]

        if query is not None:
            if isinstance(query, ustring_type):
                # Unicode queries should be pre-encoded and thus contain ascii
                # characters only
                query = query.encode('ASCII')
            elif isinstance(query, bytes):
                if kwargs:
                    if query:
                        query += ';'
                    query += make_query(kwargs)

        if (query_add or query_replace) and isinstance(query, bytes):
            query = MultiDict(parse_querystring(query.decode('ASCII')))

        if query_add is not None:
            query = query if query is not None else self.query.copy()
            query.extend(query_add)

        if query_replace is not None:
            query = query if query is not None else self.query.copy()
            query.update(query_replace)

        if query and not isinstance(query, bytes):
            query = make_query(query, **kwargs)

        url = [scheme if scheme is not None else parsed_url[0],
               netloc if netloc is not None else parsed_url[1],
               path if path is not None else parsed_url[2],
               parameters if parameters is not None else parsed_url[3],
               query,
               fragment]

        return urlunparse(url)

    def resolve_url(self, url, relative='app'):
        """
        Resolve a partially qualified URL in context of the current request.

        :param url:      A url, may be fully or partially qualified
        :param relative: One of 'app' or 'server'
        :return: A fully qualified URL

        Examples::

            >>> from fresco import FrescoApp
            >>> app = FrescoApp()
            >>> with app.requestcontext('http://example.net/foo/bar') as c:
            ...     request = c.request
            ...
            >>> request.resolve_url('/baz')
            'http://example.net/baz'
            >>> request.resolve_url('baz')
            'http://example.net/foo/baz'
            >>> request.resolve_url('http://anotherhost/bar')
            'http://anotherhost/bar'

        The returned URL path will be normalized:

            >>> app = FrescoApp()
            >>> with app.requestcontext('http://example.net/foo/bar') as c:
            ...     c.request.resolve_url('../bar')
            ...
            'http://example.net/bar'

        URLs can be resolved relative to the application's base URL (including
        SCRIPT_NAME) or the server root via the ``relative`` argument::

            >>> from fresco import FrescoApp
            >>> app = FrescoApp()
            >>> with app.requestcontext(SCRIPT_NAME='/foo',
            ...                         PATH_INFO='bar') as c:
            ...     request = c.request
            >>> request.resolve_url('/baz', relative='app')
            'http://localhost/foo/baz'
            >>> request.resolve_url('/baz', relative='server')
            'http://localhost/baz'

        If not specified, application relative paths will be assumed.
        """
        env = self.environ.get

        if '://' not in url:
            scheme = env('wsgi.url_scheme', 'http')

            if scheme == 'https':
                port = ':' + env('SERVER_PORT', '443')
            else:
                port = ':' + env('SERVER_PORT', '80')

            if scheme == 'http' and port == ':80' \
               or scheme == 'https' and port == ':443':
                port = ''

            parsed = urlparse(url)
            path_info = env('PATH_INFO', '')
            script_name = env('SCRIPT_NAME', '')

            if path_info[-1] != '/':
                path_info = path_info[:path_info.rfind('/') + 1]

            if relative == 'app':
                path = posixpath.join(quote(path_info), parsed[2])
                path = quote(script_name) + normpath(path)

            else:
                path = posixpath.join(quote(script_name) + path_info,
                                      parsed[2])

            url = urlunparse((
                env('wsgi.url_scheme', ''),
                env('HTTP_HOST', env('SERVER_NAME', '') + port),
                normpath(path),
                parsed[3],
                parsed[4],
                parsed[5],
            ))
        return url


def currentrequest():
    """\
    Return the current value of ``context.request``, or ``None`` if there is no
    request in scope.
    """
    try:
        return fresco.context.request
    except AttributeError:
        return None

from .util.http import (FileUpload, parse_querystring, parse_post,
                        get_content_type_info, get_body_bytes)



# encoding: utf-8
from copy import copy
from functools import wraps

from mock import Mock
from flea import Agent
import pytest
import tms

from fresco import FrescoApp, GET, POST, Response
from fresco.exceptions import NotFound
from fresco.compat import ustring_type
from fresco.core import urlfor
from fresco.routing import (
    Route, DelegateRoute, RouteCollection, routefor, RouteTraversal,
    RouteNotFound)
from . import fixtures


class TestMethodDispatch(object):

    def test_route_is_dispatched_to_correct_method(self):

        getview = Mock(return_value=Response(content_type='text/plain'))
        postview = Mock(return_value=Response(content_type='text/plain'))
        app = FrescoApp()
        app.route('/', GET, getview)
        app.route('/', POST, postview)

        Agent(app).get('/')
        assert getview.call_count == 1
        assert postview.call_count == 0

        Agent(app).post('/')
        assert getview.call_count == 1
        assert postview.call_count == 1


class TestRouteViewFilters(object):

    def exclaim(self, response):
        return response.replace(content=[''.join(response.content) + '!'])

    def ask(self, response):
        return response.replace(content=[''.join(response.content) + '?'])

    def test_filter_is_applied(self):
        views = fixtures.CBV('test')
        app = FrescoApp()
        app.route('/', GET, views.index_html).filter(self.ask)
        with app.requestcontext('/'):
            assert app.view().content == ['test?']

    def test_it_applies_multiple_filters_in_order(self):
        app = FrescoApp()
        views = fixtures.CBV('test')
        app.route('/', GET, views.index_html).filter(self.ask, self.exclaim)
        with app.requestcontext('/'):
            assert app.view().content == ['test?!']

    def test_it_applies_chained_filter_calls_in_order(self):
        app = FrescoApp()
        views = fixtures.CBV('test')
        app.route('/', GET, views.index_html)\
                .filter(self.ask).filter(self.exclaim)
        with app.requestcontext('/'):
            assert app.view().content == ['test?!']


class TestRouteDecorators(object):

    def exclaim(self, func):
        @wraps(func)
        def exclaim(*args, **kwargs):
            response = func(*args, **kwargs)
            return response.replace(content=[''.join(response.content) + '!'])
        return exclaim

    def test_decorator_is_applied(self):

        views = fixtures.CBV('test')

        app = FrescoApp()
        app.route('/decorated', GET, views.index_html,
                  decorators=[self.exclaim])
        app.route('/plain', GET, views.index_html)

        with app.requestcontext('/decorated'):
            assert app.view().content == ['test!']

        with app.requestcontext('/plain'):
            assert app.view().content == ['test']

    def test_decorator_is_applied_with_wrap_method(self):

        views = fixtures.CBV('test')

        app = FrescoApp()
        app.route('/decorated', GET, views.index_html).wrap(self.exclaim)
        app.route('/plain', GET, views.index_html)

        with app.requestcontext('/decorated'):
            assert app.view().content == ['test!']

        with app.requestcontext('/plain'):
            assert app.view().content == ['test']

    def test_decorator_works_with_urlfor(self):

        views = fixtures.CBV('test')
        app = FrescoApp()
        app.route('/decorated', GET, views.index_html,
                  decorators=[self.exclaim])
        with app.requestcontext():
            assert urlfor(views.index_html, _app=app) == \
                'http://localhost/decorated'

    def test_using_wraps_with_viewspec_doesnt_raise_AttributeError(self):

        def decorator(func):
            @wraps(func)
            def decorator(*args, **kwargs):
                return func(*args, **kwargs)
            return decorator

        class Views(object):
            __routes__ = Route('/',
                               GET, 'index_html', decorators=[decorator]),

            def index_html(self):
                return Response(['hello'])

        app = FrescoApp()
        app.include('/', Views())


class TestPredicates(object):

    def test_predicate_match(self):

        def v1():
            return Response(['x'])

        def v2():
            return Response(['y'])

        app = FrescoApp()
        app.route('/', GET, v1, predicate=lambda request: 'x' in request.query)
        app.route('/', GET, v2, predicate=lambda request: 'y' in request.query)

        t = Agent(app)
        assert t.get('/?x=1', check_status=False).body == 'x'
        assert t.get('/?y=1', check_status=False).body == 'y'
        assert t.get('/', check_status=False).response.status_code == 404


class TestRouteNames(object):

    def test_name_present_in_route_keys(self):
        r = Route('/', GET, None, name='foo')
        assert 'foo' in list(r.route_keys())

    def test_name_with_other_kwargs(self):
        r = Route('/', GET, None, name='foo', x='bar')
        assert 'foo' in list(r.route_keys())

    def test_name_cannot_contain_colon(self):
        with pytest.raises(ValueError):
            Route('/', GET, None, name='foo:bar')


class TestRouteCollection(object):

    def test_get_routes_matches_on_method(self):

        r_get = Route('/', GET, None)
        r_post = Route('/', POST, None)

        rc = RouteCollection([r_post, r_get])

        assert [r.route for r in rc.get_routes('/', GET)] == [r_get]
        assert [r.route for r in rc.get_routes('/', POST)] == [r_post]

    def test_get_routes_matches_on_path(self):

        r1 = Route('/1', GET, None)
        r2 = Route('/2', GET, None)

        rc = RouteCollection([r1, r2])

        assert [r.route for r in rc.get_routes('/1', GET)] == [r1]
        assert [r.route for r in rc.get_routes('/2', GET)] == [r2]

    def test_get_routes_can_match_all_methods(self):

        r1 = Route('/1', GET, None)
        r2 = Route('/1', POST, None)

        rc = RouteCollection([r1, r2])
        assert [r.route for r in rc.get_routes('/1', None)] == [r1, r2]

    def test_route_returns_traversal_information_on_nested_routes(self):

        a = RouteCollection()
        b = RouteCollection()

        a.route('/harvey', GET, lambda: None)
        b.route('/harvey', GET, lambda: None)

        a.delegate('/rabbit', b)
        b.delegate('/hole', a)

        r = next(a.get_routes('/rabbit/hole/rabbit/harvey', None))
        assert r.collections_traversed == [(a, ''),
                                           (b, '/rabbit'),
                                           (a, '/rabbit/hole'),
                                           (b, '/rabbit/hole/rabbit')]

    def test_pathfor_works_with_positional_args(self):
        view = Mock()
        rc = RouteCollection([Route('/<:str>', GET, view)])
        assert rc.pathfor(view, 'x') == '/x'

    def test_replace_raises_route_not_found(self):
        a = RouteCollection()
        view = Mock()
        a.route('/harvey', GET, view, name='harvey')
        with pytest.raises(RouteNotFound):
            a.replace('rabbit', None)

    def test_replace_selects_routes_by_name(self):
        a = RouteCollection()
        oldroute = Route('/', GET, Mock(), name='harvey')
        newroute = Route('/', GET, Mock())
        a.add_route(oldroute)
        a.replace('harvey', newroute)
        assert a.__routes__ == [newroute]

    def test_replace_selects_routes_by_view(self):
        a = RouteCollection()
        view = Mock()
        oldroute = Route('/', GET, view)
        newroute = Route('/', GET, Mock())
        a.add_route(oldroute)
        a.replace(view, newroute)
        assert a.__routes__ == [newroute]

    def test_can_add_a_list_to_a_routecollection(self):
        r1 = Route('/', GET, Mock())
        r2 = Route('/', GET, Mock())
        assert (RouteCollection([r1]) + [r2]).__routes__ == [r1, r2]

    def test_can_add_routecollections(self):
        r1 = Route('/', GET, Mock())
        r2 = Route('/', GET, Mock())
        assert (RouteCollection([r1]) + RouteCollection([r2])).__routes__ == \
                [r1, r2]

    def test_routecollections_can_be_used_in_classes(self):
        class MyViews(object):
            __routes__ = RouteCollection([
                Route('/', GET, 'view')])

            def view(self):
                pass

        v = MyViews()
        app = FrescoApp()
        app.include('/', v)
        assert [r.route.getview()
                for r in app.get_routes('/', GET)] == [v.view]

    def test_routecollections_in_classes_can_be_manipulated(self):
        class MyViews(object):
            __routes__ = RouteCollection([
                Route('/', GET, 'view')])

            def view(self):
                pass

        class MyOtherViews(MyViews):
            __routes__ = copy(MyViews.__routes__)
            __routes__.replace('view', Route('/', GET, 'another_view'))

            def another_view(self):
                pass

        v = MyOtherViews()
        app = FrescoApp()
        app.include('/', v)
        assert [r.route.getview()
                for r in app.get_routes('/', GET)] == [v.another_view]


class TestRoutefor(object):

    def test_routefor_with_view_function(self):

        def view():
            return Response(['ok'])

        app = FrescoApp()
        route = app.route('/foo', GET, view)

        with app.requestcontext():
            assert routefor(view) == route

    def test_routefor_with_string(self):
        app = FrescoApp()
        route = app.route('/myviewfunc', GET, fixtures.module_level_function)
        with app.requestcontext():
            assert routefor('fresco.tests.fixtures.module_level_function') == \
                route

    def test_routefor_generates_first_route(self):

        myviewfunc = lambda req: Response([])
        app = FrescoApp()
        r1 = app.route('/1', GET, myviewfunc)
        app.route('/2', GET, myviewfunc)
        with app.requestcontext():
            assert routefor(myviewfunc) == r1


class TestDelegatedRoutes(object):

    def test_dispatch_to_delegated_route(self):

        def hello():
            return Response(['hello'])

        inner = FrescoApp()
        inner.route('/hello', GET, hello)

        outer = FrescoApp()
        outer.delegate('/say', inner)

        r = Agent(outer).get('/say/hello')
        assert r.body == 'hello'

    def test_url_variables_are_passed(self):

        hello = Mock()

        inner = FrescoApp()
        inner.route('/<i:str>', GET, hello)

        outer = FrescoApp()
        outer.delegate('/<o:str>', inner)

        with outer.requestcontext('/foo/bar'):
            outer.view()
            hello.assert_called_with(i='bar', o='foo')

    def test_delegation_to_dynamic_routes(self):

        result = []

        class MyRoutes(object):
            __routes__ = [Route('/<inner:int>/view', GET, 'view')]

            def __init__(self, **kwargs):
                self.kwargs = kwargs

            def view(self, **kwargs):
                result.append((self, kwargs))

        app = FrescoApp()
        app.delegate('/<outer:str>', MyRoutes, dynamic=True)
        with app.requestcontext('/two/2/view'):
            app.view()
            instance, inner_kwargs = result[0]
            assert inner_kwargs == {'inner': 2}
            assert instance.kwargs == {'outer': 'two'}

    def test_dynamic_routes_are_never_shared(self):

        result = []

        class MyRoutes(object):
            __routes__ = [Route('', GET, 'view')]

            def __init__(self, value):
                self.value = value

            def view(self):
                result.append(self.value)

        app = FrescoApp()
        app.delegate('/<value:str>', MyRoutes, dynamic=True)
        with app.requestcontext('/one'):
            app.view()
            v1 = result.pop()
        with app.requestcontext('/two'):
            app.view()
            v2 = result.pop()
        assert v1 == 'one'
        assert v2 == 'two', v2

    def test_pathfor_with_delegated_route(self):
        inner = FrescoApp()
        inner.route('/<i:str>', GET, lambda: None, name='inner-route')

        outer = FrescoApp()
        outer.delegate('/<o:str>', inner, name='delegation')

        with outer.requestcontext('/foo/bar'):
            assert outer.pathfor('delegation:inner-route',
                                 o='x', i='y') == '/x/y'

    def test_pathfor_with_dynamic_delegated_route(self):

        view = Mock()

        def routecollectionfactory(*args, **kwargs):
            return RouteCollection([Route('/<i:str>', GET, view,
                                          name='inner-route')])

        rc = RouteCollection()
        rc.delegate('/<o:str>', routecollectionfactory,
                    name='delegation', dynamic=True)

        assert rc.pathfor('delegation:inner-route', o='x', i='y') == '/x/y'

    def test_pathfor_with_dynamic_delegated_route_uses_default_args(self):

        view = Mock()

        def routecollectionfactory(factoryarg1, factoryarg2):
            return RouteCollection([Route('/<i:str>', GET, view,
                                          name='inner-route')])

        rc = RouteCollection()
        rc.delegate('/<factoryarg1:str>/<factoryarg2:str>',
                    routecollectionfactory,
                    factoryarg1_default='foo',
                    factoryarg2_default=lambda r: 'bar',
                    name='delegation', dynamic=True)

        assert rc.pathfor('delegation:inner-route', i='y') == '/foo/bar/y'

    def test_urlfor_with_dynamic_delegated_route_and_view_self(self):

        result = []

        class MyRoutes(object):
            __routes__ = [Route('/<inner:int>/view', GET, 'view')]

            def __init__(self, **kwargs):
                self.kwargs = kwargs

            def view(self, **kwargs):
                result.append(urlfor(self.view, inner=3))

        app = FrescoApp()
        app.delegate('/<outer:str>', MyRoutes, dynamic=True)
        with app.requestcontext('/two/2/view'):
            app.view()
            assert result == ['http://localhost/two/3/view']

    def test_urlgeneration_with_dynamic_routes(self):

        class Routable(object):
            __routes__ = [Route('/<b:int>', GET, 'view', name='y')]

            def __init__(self, a):
                pass

            def view(self, b):
                pass

        app = FrescoApp()
        app.delegate('/<a:str>', Routable, dynamic=True, name='x')
        with app.requestcontext('/two/2/view'):
            assert urlfor('x:y', a='a', b=1) == 'http://localhost/a/1'

    def test_delegated_routes_can_be_included(self):

        view = Mock()

        inner = RouteCollection([Route('/baz', GET, view)])
        middle = RouteCollection([DelegateRoute('/bar', inner)])
        outer = FrescoApp()
        outer.include('/foo', middle)
        with outer.requestcontext('/foo/bar/baz'):
            outer.view()
            view.assert_called()

    def test_not_found_is_returned(self):

        def inner():
            raise NotFound()

        outer = FrescoApp()
        outer.delegate('/foo', inner, dynamic=True)
        with outer.requestcontext('/foo/bar/baz'):
            response = outer.view()
            assert response.status_code == 404

    def test_not_found_causes_next_route_to_be_tried(self):

        def inner():
            raise NotFound()
        view = Mock()

        outer = FrescoApp()
        outer.delegate('/foo', inner, dynamic=True)
        outer.route('/foo', view)
        with outer.requestcontext('/foo'):
            outer.view()
            view.assert_called()


class TestConverters(object):

    def test_str_converter_returns_unicode(self):
        from fresco.routing import StrConverter
        s = ustring_type('abc')
        assert isinstance(StrConverter().from_string(s), ustring_type)

    def test_register_coverter_acts_as_decorator(self):
        from fresco.routing import Converter, register_converter

        @register_converter('testconverter')
        class MyConverter(Converter):
            def from_string(self, s):
                return 'bar'

        view = Mock()

        app = FrescoApp()
        app.route('/<:testconverter>', GET, view)
        with app.requestcontext('/foo'):
            app.view()
            assert view.call_args == (('bar',), {}), view.call_args


class TestViewArgs(object):

    def test_it_uses_args(self):
        routes = RouteCollection([Route('/', GET, None, args=(1, 2))])
        assert list(routes.get_routes('/', GET)) == \
                [tms.InstanceOf(RouteTraversal, args=(1, 2))]

    def test_it_uses_view_args(self):
        routes = RouteCollection([Route('/', GET, None, view_args=(1, 2))])
        assert list(routes.get_routes('/', GET)) == \
                [tms.InstanceOf(RouteTraversal, args=(1, 2))]

    def test_it_appends_args_extracted_from_path(self):
        routes = RouteCollection([Route('/<:int>', GET, None,
                                        view_args=(1, 2))])
        assert list(routes.get_routes('/3', GET)) == \
                [tms.InstanceOf(RouteTraversal, args=(1, 2, 3))]


class TestViewKwargs(object):

    def test_it_reads_from_route_kwargs(self):
        routes = RouteCollection([Route('/', GET, None, x=1)])
        assert list(routes.get_routes('/', GET)) == \
                [tms.InstanceOf(RouteTraversal, kwargs={'x': 1})]

    def test_it_reads_from_kwargs(self):
        routes = RouteCollection([Route('/', GET, None, kwargs={'x': 1})])
        assert list(routes.get_routes('/', GET)) == \
                [tms.InstanceOf(RouteTraversal, kwargs={'x': 1})]

    def test_it_reads_from_view_kwargs(self):
        routes = RouteCollection([Route('/', GET, None, view_kwargs={'x': 1})])
        assert list(routes.get_routes('/', GET)) == \
                [tms.InstanceOf(RouteTraversal, kwargs={'x': 1})]


class TestRouteClassIsPluggable(object):

    class CustomRoute(Route):
        pass

    def test_it_defaults_to_Route(self):
        routes = RouteCollection()
        assert routes.route_class is Route

    def test_it_accepts_route_class_arg(self):
        routes = RouteCollection(route_class=self.CustomRoute)
        assert routes.route_class is self.CustomRoute

    def test_it_uses_route_class_in_route_method(self):

        def myview():
            pass

        routes = RouteCollection(route_class=self.CustomRoute)
        routes.route('/', GET, myview)

        assert list(routes.get_routes('/', GET)) == \
                [tms.InstanceOf(RouteTraversal,
                                route=tms.InstanceOf(self.CustomRoute))]

    def test_it_uses_route_class_in_decorator(self):

        routes = RouteCollection(route_class=self.CustomRoute)

        @routes.route('/', GET)
        def myview():
            pass

        assert list(routes.get_routes('/', GET)) == \
                [tms.InstanceOf(RouteTraversal,
                                route=tms.InstanceOf(self.CustomRoute))]

    def test_custom_route_class_survives_include(self):
        routes = RouteCollection(route_class=self.CustomRoute)

        @routes.route('/', GET)
        def myview():
            pass

        routes2 = RouteCollection()
        routes2.include('/incl', routes)

        assert list(routes2.get_routes('/incl/', GET)) == \
                [tms.InstanceOf(RouteTraversal,
                                route=tms.InstanceOf(self.CustomRoute))]



from flea import Agent
from fresco.util.wsgi import ClosingIterator, StartResponseWrapper


class Counter(object):

    value = 0

    def inc(self):
        self.value += 1


def start_response(status, headers, exc_info=None):
    pass


class _TestException(Exception):
    pass


class TestClosingIterator(object):

    def app(self, environ, start_response):
        start_response('200 OK', [('Content-Type: text/plain')])
        yield "Foo"
        yield "Bar"

    def test_close_called_after_iterator_finished(self):
        count = Counter()
        environ = {}

        result = self.app(environ, start_response)
        result = ClosingIterator(result, count.inc)
        assert count.value == 0
        try:
            list(result)
        finally:
            result.close()
        assert count.value == 1

    def test_multiple_close_functions_called(self):
        count1 = Counter()
        count2 = Counter()
        environ = {}

        result = self.app(environ, start_response)
        result = ClosingIterator(result, count1.inc, count2.inc)
        assert count1.value == 0
        assert count2.value == 0
        try:
            list(result)
        finally:
            result.close()
        assert count1.value == 1
        assert count2.value == 1


class TestStartResponseWrapper(object):

    def test_write(self):

        def wsgiapp(environ, start_response):
            start_response = StartResponseWrapper(start_response)
            write = start_response('200 OK', [('Content-Type', 'text/plain')])

            write(b'cat')
            write(b'sat')
            write2 = start_response.call_start_response()
            write2(b'mat')
            return []

        assert Agent(wsgiapp).get('/').body == 'catsatmat'



"""
An order preserving multidict implementation

"""

try:
    from collections import MutableMapping
except ImportError:
    from UserDict import DictMixin

    class MutableMapping(DictMixin):
        """
        Emulate MutableMapping in Python<2.6
        """

from itertools import repeat


class MultiDict(MutableMapping):
    """
    Dictionary-like object that supports multiple values per key. Insertion
    order is preserved.

    Synopsis:

        >>> from fresco.multidict import MultiDict
        >>> d = MultiDict([('a', 1), ('a', 2), ('b', 3)])
        >>> d['a']
        1
        >>> d['b']
        3
        >>> d.getlist('a')
        [1, 2]
        >>> d.getlist('b')
        [3]
        >>> list(d.allitems())
        [('a', 1), ('a', 2), ('b', 3)]

    """

    setdefault = MutableMapping.setdefault
    __contains__ = MutableMapping.__contains__

    def __init__(self, *args, **kwargs):
        """
        MultiDicts can be constructed in the following ways:

            1. From a sequence of ``(key, value)`` pairs:

                >>> from fresco.multidict import MultiDict
                >>> MultiDict([('a', 1), ('a', 2)])  # doctest: +ELLIPSIS
                MultiDict(...)

            2. Initialized from another MultiDict:

                >>> from fresco.multidict import MultiDict
                >>> d = MultiDict([('a', 1), ('a', 2)])
                >>> MultiDict(d)  # doctest: +ELLIPSIS
                MultiDict(...)

            3. Initialized from a regular dict:

                >>> from fresco.multidict import MultiDict
                >>> MultiDict({'a': 1})  # doctest: +ELLIPSIS
                MultiDict(...)

            4. From keyword arguments:

                >>> from fresco.multidict import MultiDict
                >>> MultiDict(a=1)  # doctest: +ELLIPSIS
                MultiDict(...)

        """
        if len(args) > 1:
            raise TypeError("%s expected at most 2 arguments, got %d" %
                            (self.__class__.__name__,  1 + len(args)))
        self.clear()
        self.update(*args, **kwargs)

    def __getitem__(self, key):
        """
        Return the first item associated with ``key``:

            >>> d = MultiDict([('a', 1), ('a', 2)])
            >>> d['a']
            1
        """
        try:
            return self._dict[key][0]
        except IndexError:
            raise KeyError(key)

    def __setitem__(self, key, value):
        """
        Set the items associated with key to a single item, ``value``.

            >>> d = MultiDict()
            >>> d['b'] = 3
            >>> d
            MultiDict([('b', 3)])
        """
        _order = [(k, v) for k, v in self._order if k != key]
        _order.append((key, value))
        self._order = _order
        self._dict[key] = [value]

    def __delitem__(self, key):
        """
        Delete all values associated with ``key``
        """
        del self._dict[key]
        self._order = [(k, v) for (k, v) in self._order if k != key]

    def __iter__(self):
        """
        Return an iterator over all keys
        """
        return (k for k in self._dict)

    def get(self, key, default=None):
        """
        Return the first available value for key ``key``, or ``default`` if no
        such value exists:

            >>> d = MultiDict([('a', 1), ('a', 2)])
            >>> print(d.get('a'))
            1
            >>> print(d.get('b'))
            None
        """
        return self._dict.get(key, [default])[0]

    def getlist(self, key):
        """
        Return a list of all values for key ``key``.
        """
        return self._dict.get(key, [])

    def copy(self):
        """\
        Return a shallow copy of the dictionary:

            >>> d = MultiDict([('a', 1), ('a', 2), ('b', 3)])
            >>> copy = d.copy()
            >>> copy
            MultiDict([('a', 1), ('a', 2), ('b', 3)])
            >>> copy is d
            False
        """
        return self.__class__(self)

    def fromkeys(cls, seq, value=None):
        """\
        Create a new MultiDict with keys from seq and values set to value.

        Example:

            >>> MultiDict.fromkeys(['a', 'b'])
            MultiDict([('a', None), ('b', None)])

        Keys can be repeated:

            >>> d = MultiDict.fromkeys(['a', 'b', 'a'])
            >>> d.getlist('a')
            [None, None]
            >>> d.getlist('b')
            [None]

        """
        return cls(zip(seq, repeat(value)))
    fromkeys = classmethod(fromkeys)

    def items(self):
        """\
        Return a list of ``(key, value)`` tuples. Only the first ``(key,
        value)`` is returned where keys have multiple values:

            >>> d = MultiDict([('a', 1), ('a', 2), ('b', 3)])
            >>> list(d.items())
            [('a', 1), ('b', 3)]
        """
        seen = set()
        for k, v in self._order:
            if k in seen:
                continue
            yield k, v
            seen.add(k)

    def listitems(self):
        """\
        Like ``items``, but returns lists of values:

            >>> d = MultiDict([('a', 1), ('a', 2), ('b', 3)])
            >>> list(d.listitems())
            [('a', [1, 2]), ('b', [3])]
        """
        for k in self.keys():
            yield k, self._dict[k]

    def allitems(self):
        """\
        Return ``(key, value)`` pairs for each item in the MultiDict.
        Items with multiple keys will have multiple key-value pairs returned:

            >>> from fresco.multidict import MultiDict
            >>> d = MultiDict([('a', 1), ('a', 2), ('b', 3)])
            >>> list(d.allitems())
            [('a', 1), ('a', 2), ('b', 3)]
        """
        return iter(self._order)

    def keys(self):
        """\
        Return dictionary keys. Each key is returned only once, even if
        multiple values are present.

            >>> from fresco.multidict import MultiDict
            >>> d = MultiDict([('a', 1), ('a', 2), ('b', 3)])
            >>> list(d.keys())
            ['a', 'b']
        """
        return (k for k, v in self.items())

    def values(self):
        """\
        Return values from the dictionary. Where keys have multiple values,
        only the first is returned:

            >>> from fresco.multidict import MultiDict
            >>> d = MultiDict([('a', 1), ('a', 2), ('b', 3)])
            >>> list(d.values())
            [1, 3]
        """
        return (v for k, v in self.items())

    def listvalues(self):
        """\
        Return an iterator over lists of values. Each item will be the list of
        values associated with a single key.

        Example usage:

            >>> from fresco.multidict import MultiDict
            >>> d = MultiDict([('a', 1), ('a', 2), ('b', 3)])
            >>> list(d.listvalues())
            [[1, 2], [3]]
        """
        return (self._dict[k] for k in self.keys())

    def pop(self, key, *args):
        """
        Dictionary ``pop`` method. Return and remove the value associated with
        ``key``. If more than one value is associated with ``key``, only the
        first is returned.

        Example usage:

            >>> d = MultiDict([('a', 1), ('a', 2), ('b', 3)])
            >>> d.pop('a')
            1
            >>> d
            MultiDict([('a', 2), ('b', 3)])
            >>> d.pop('a')
            2
            >>> d
            MultiDict([('b', 3)])
            >>> d.pop('a')
            Traceback (most recent call last):
                ...
            KeyError: 'a'
        """
        if len(args) > 1:
            raise TypeError("pop expected at most 2 arguments, got %d" %
                            (1 + len(args)))
        try:
            value = self._dict[key].pop(0)
        except (KeyError, IndexError):
            if args:
                return args[0]
            raise KeyError(key)
        self._order.remove((key, value))
        return value

    def popitem(self):
        """
        Return and remove a ``(key, value)`` pair from the dictionary.

        The item popped is always the most recently added key and the first
        value associated with it:

            >>> d = MultiDict([('a', 1), ('a', 2), ('b', 3)])
            >>> d.popitem()
            ('b', 3)
            >>> d.popitem()
            ('a', 1)
            >>> d.popitem()
            ('a', 2)
            >>> d.popitem() #doctest: +ELLIPSIS
            Traceback (most recent call last):
                ...
            KeyError: 'popitem(): dictionary is empty'
        """
        try:
            key = self._order[-1][0]
        except IndexError:
            raise KeyError('popitem(): dictionary is empty')
        return key, self.pop(key)

    def update(self, *args, **kwargs):
        r"""
        Update the MultiDict from another MultiDict, regular dictionary or a
        iterable of ``(key, value)`` pairs. New keys overwrite old keys - use
        :meth:`extend` if you want new keys to be added to old keys.

        Updating from another MultiDict:

            >>> d = MultiDict([('name', 'eric'), ('occupation', 'lumberjack')])
            >>> d.update(MultiDict([('mood', 'okay')]))
            >>> d
            MultiDict([('name', 'eric'), ('occupation', 'lumberjack'), ('mood', 'okay')])

        from a dictionary:

            >>> d = MultiDict([('name', 'eric'), ('hobby', 'shopping')])
            >>> d.update({'hobby': 'pressing wild flowers'})
            >>> d
            MultiDict([('name', 'eric'), ('hobby', 'pressing wild flowers')])

        an iterable of ``(key, value)`` pairs:

            >>> d = MultiDict([('name', 'eric'), ('occupation', 'lumberjack')])
            >>> d.update([('hobby', 'shopping'),
            ...           ('hobby', 'pressing wild flowers')])
            >>> d
            MultiDict([('name', 'eric'), ('occupation', 'lumberjack'), ('hobby', 'shopping'), ('hobby', 'pressing wild flowers')])

        or keyword arguments:

                >>> d = MultiDict([('name', 'eric'),\
                ...                ('occupation', 'lumberjack')])
                >>> d.update(mood='okay')

        """
        if len(args) > 1:
            raise TypeError("expected at most 1 argument, got %d" %
                            (1 + len(args),))
        if args:
            other = args[0]
        else:
            other = []
        return self._update(other, True, **kwargs)

    def extend(self, *args, **kwargs):
        """
        Extend the MultiDict with another MultiDict, regular dictionary or a
        iterable of ``(key, value)`` pairs. This is similar to :meth:`update`
        except that new keys are added to old keys.
        """
        if len(args) > 1:
            raise TypeError("expected at most 1 argument, got %d"\
                            (1 + len(args),))
        if args:
            other = args[0]
        else:
            other = []
        return self._update(other, False, **kwargs)

    def _update(self, *args, **kwargs):
        """
        Update the MultiDict from another object and optionally kwargs.

        :param other: the other MultiDict, dict, or iterable (first positional
                      arg)
        :param replace: if ``True``, entries will replace rather than extend
                        existing entries (second positional arg)
        """
        other, replace = args

        if isinstance(other, self.__class__):
            items = list(other.allitems())
        elif isinstance(other, dict):
            items = list(other.items())
        else:
            items = list(other)

        if kwargs:
            items += list(kwargs.items())

        if replace:
            replaced = set(k for k, v in items if k in self._dict)
            self._order = [(k, v)
                           for (k, v) in self._order if k not in replaced]
            for key in replaced:
                self._dict[key] = []

        for k, v in items:
            self._dict.setdefault(k, []).append(v)
            self._order.append((k, v))

    def __repr__(self):
        """
        ``__repr__`` representation of object
        """
        return '%s(%r)' % (self.__class__.__name__, list(self.allitems()))

    def __str__(self):
        """
        ``__str__`` representation of object
        """
        return repr(self)

    def __len__(self):
        """
        Return the total number of keys stored.
        """
        return len(self._dict)

    def __eq__(self, other):
        return isinstance(other, MultiDict) \
            and self._order == other._order

    def __ne__(self, other):
        return not (self == other)

    def clear(self):
        self._order = []
        self._dict = {}



from fresco.util.common import object_or_404
from fresco.exceptions import NotFound
import pytest


class TestObjectOr404(object):

    def test_it_returns_the_object(self):
        ob = object()
        assert object_or_404(ob) is ob

    def test_it_raises_NotFound(self):
        with pytest.raises(NotFound):
            object_or_404(None)

    def test_it_raises_custom_exception(self):
        class Foo(Exception):
            pass
        with pytest.raises(Foo):
            object_or_404(None, exception=Foo)



from mock import Mock
import pytest

from fresco import Response
from fresco import FrescoApp, GET, POST, PUT, DELETE
from fresco import Route, urlfor, context
from fresco.util.wsgi import ClosingIterator
from . import fixtures


class TestFrescoApp(object):

    def test_route_operates_as_a_decorator(self):

        app = FrescoApp()

        @app.route('/', GET)
        def view():
            return Response(['ok'])

        with app.requestcontext('/'):
            assert app.view().content == ['ok']

    def test_route_decorator_sets_url_property(self):
        app = FrescoApp()

        @app.route('/', GET)
        def view():
            return Response(['ok'])

        assert callable(view.url)

    def test_route_operates_as_a_function(self):

        def view():
            return Response(['ok'])

        app = FrescoApp()
        app.route('/', GET, view)
        with app.requestcontext('/'):
            assert app.view().content == ['ok']

    def test_route_returns_route_instance(self):
        def view():
            return Response(['ok'])

        app = FrescoApp()
        assert isinstance(app.route('/', GET, view), Route)

    def test_route_http_methods(self):

        def view():
            return Response([context.request.environ['REQUEST_METHOD']])

        app = FrescoApp()
        app.route('/get', GET, view)
        app.route('/post', POST, view)

        with app.requestcontext('/get', REQUEST_METHOD='GET'):
            assert app.view().status_code == 200

        with app.requestcontext('/get', REQUEST_METHOD='POST'):
            assert app.view().status_code == 405

        with app.requestcontext('/post', REQUEST_METHOD='GET'):
            assert app.view().status_code == 405

        with app.requestcontext('/post', REQUEST_METHOD='POST'):
            assert app.view().status_code == 200

    def test_HEAD_request_delegated_to_GET_view(self):

        app = FrescoApp()

        @app.route('/', GET)
        def view():
            return Response([], x_original_view='GET')

        with app.requestcontext('/', REQUEST_METHOD='HEAD'):
            assert app.view().get_header('X-Original-View') == 'GET'

    def test_NotFound_observed_when_raised_in_handler(self):

        def app1():
            from fresco.exceptions import NotFound
            if 'foo' in context.request.path_info:
                raise NotFound()
            return Response(['app1'])

        def app2():
            return Response(['app2'])

        app = FrescoApp()
        app.route_all('/', GET, app1)
        app.route_all('/', GET, app2)

        with app.requestcontext('/bar'):
            assert app.view().content == ['app1']

        with app.requestcontext('/foo'):
            assert app.view().content == ['app2']

    def test_NotFound_final_observed_when_raised_in_handler(self):

        def app1():
            from fresco.exceptions import NotFound
            if 'foo' in context.request.path_info:
                raise NotFound(final=True)
            return Response(['app1'])

        def app2():
            return Response(['app2'])

        app = FrescoApp()
        app.route_all('/', GET, app1)
        app.route_all('/', GET, app2)

        with app.requestcontext('/bar'):
            assert app.view().content == ['app1']

        with app.requestcontext('/foo/'):
            assert app.view().status_code == 404

    def test_apps_called_in_correct_order(self):

        def view(value=''):
            return Response([value])

        app = FrescoApp()
        app.route_all('/f', GET, view, value='foo')
        app.route_all('/', GET, view, value='bar')

        with app.requestcontext('/f/bar'):
            assert app.view().content == ['foo']

        with app.requestcontext('/b/bar'):
            assert app.view().content == ['bar']

    def test_wsgi_app_handles_response_exceptions(self):

        from fresco.exceptions import NotFound

        def view():
            raise NotFound()

        app = FrescoApp()
        app.route('/', GET, view)

        with app.requestcontext('/'):
            assert app.view().status_code == 404

    def test_route_wsgi_app(self):

        def wsgiapp(environ, start_response):

            start_response('200 OK',
                           [('Content-Type', 'application/x-pachyderm')])
            return ['pretty pink elephants']

        app = FrescoApp()
        app.route_wsgi('/', wsgiapp)

        with app.requestcontext('/'):
            assert app.view().content == ['pretty pink elephants']
            assert app.view().get_header('Content-Type') == \
                    'application/x-pachyderm'

    def test_get_methods_matches_on_path(self):

        app = FrescoApp()
        app.route('/1', POST, lambda: None)
        app.route('/1', PUT, lambda: None)
        app.route('/2', GET, lambda: None)
        app.route('/2', DELETE, lambda: None)

        with app.requestcontext() as c:
            assert app.get_methods(app, c.request, '/1') == set([POST, PUT])

        with app.requestcontext() as c:
            assert app.get_methods(app, c.request, '/2') == set([GET, DELETE])

    def test_get_methods_matches_on_predicate(self):

        p1 = Mock(return_value=True)
        p2 = Mock(return_value=False)

        app = FrescoApp()
        app.route('/', POST, lambda: None, predicate=p1)
        app.route('/', PUT, lambda: None, predicate=p2)

        with app.requestcontext('/') as c:
            assert app.get_methods(app, c.request, '/') == set([POST])
            p1.assert_called_with(c.request)
            p2.assert_called_with(c.request)

    def test_invalid_path_encoding_triggers_bad_request(self):
        app = FrescoApp()
        with app.requestcontext(
                PATH_INFO=fixtures.misquoted_wsgi_unicode_path):
            assert app.view().status_code == 400


class TestIncludeApp(object):

    def test_it_routes_to_an_included_app(self):

        app = FrescoApp()

        @app.route('/', GET)
        def view():
            return Response(['ok'])

        app2 = FrescoApp()
        app2.include('/app1', app)

        with app2.requestcontext('/'):
            assert app2.view().status_code == 404

        with app2.requestcontext('/app1/'):
            assert app2.view().content == ['ok']

    def test_included_app_can_use_urlfor(self):

        def view():
            url = urlfor(view)
            return Response(url)

        app = FrescoApp()
        app.route('/', GET, view)
        app2 = FrescoApp()
        app2.include('/app1', app)

        with app2.requestcontext('/app1/'):
            assert app2.view().content == 'http://localhost/app1/'


class TestTrailingSlashes(object):
    """\
    The general principle is that if a GET or HEAD request is received for a
    URL without a trailing slash and no match is found, the app will look for a
    URL with a trailing slash, and redirect the client if such a route exists.
    """

    def test_no_trailing_slash(self):

        def foo():
            return Response(['foo'])

        app = FrescoApp()
        app.route('/foo/', GET, foo)

        with app.requestcontext('/foo'):
            assert app.view().status_code == 301
            assert app.view().get_header('location') == 'http://localhost/foo/'


class TestViewCollection(object):

    def test_appdef(self):

        app = FrescoApp()
        app.include('/', fixtures.CBV('bananas!'))
        with app.requestcontext('/'):
            assert app.view().content == ['bananas!']

    def test_appdef_url_generation(self):

        foo = fixtures.CBV('foo!')
        bar = fixtures.CBV('bar!')
        baz = fixtures.CBV('baz!')

        app = FrescoApp(foo)
        app.include('/bar', bar)
        app.include('/baz', baz)

        with app.requestcontext():
            assert urlfor(foo.index_html) == 'http://localhost/'
            assert urlfor(bar.index_html) == 'http://localhost/bar/'
            assert urlfor(baz.index_html) == 'http://localhost/baz/'

    def test_instance_available_in_context(self):

        s = []

        class MyCBV(fixtures.CBV):

            def index_html(self):
                from fresco import context
                s.append(context.view_self)
                return Response([])

        instance = MyCBV('foo!')
        app = FrescoApp(instance)

        with app.requestcontext('/'):
            app.view()
            assert s[0] is instance


class TestContextAttributes(object):

    def test_app_is_set(self):

        def check_app(expected):
            assert context.app is expected
            return Response([])

        app1 = FrescoApp()
        app2 = FrescoApp()

        app1.route('/', GET, check_app, {'expected': app1})
        app2.route('/', GET, check_app, {'expected': app2})

        with app1.requestcontext('/'):
            app1.view()

        with app2.requestcontext('/'):
            app2.view()


class TestAppRequestContext(object):

    def _middleware(self, app):
        def middleware(environ, start_response):
            environ['sausages'] = 1
            return app(environ, start_response)
        return middleware

    def test_creates_isolated_context(self):

        app = FrescoApp()
        with app.requestcontext():
            context.request = 'foo'

            with app.requestcontext():
                context.request = 'bar'
                assert context.request == 'bar'

            assert context.request == 'foo'

    def test_parses_full_url(self):

        with FrescoApp().requestcontext('https://arthur@example.org:123/?x=y'):
            assert context.request.environ['HTTPS'] == 'on'
            assert context.request.environ['REMOTE_USER'] == 'arthur'
            assert context.request.environ['HTTP_HOST'] == 'example.org:123'
            assert context.request.environ['SCRIPT_NAME'] == ''
            assert context.request.environ['PATH_INFO'] == '/'
            assert context.request.environ['QUERY_STRING'] == 'x=y'

    def test_it_posts_data(self):
        with FrescoApp().requestcontext_post(data={'foo': 'bar'}) as c:
            assert c.request.method == 'POST'
            assert list(c.request.form.allitems()) == [('foo', 'bar')]
            assert list(c.request.query.allitems()) == []

    def test_it_puts_data(self):
        with FrescoApp().requestcontext_put(data={'foo': 'bar'}) as c:
            assert c.request.method == 'PUT'
            assert list(c.request.form.allitems()) == [('foo', 'bar')]

    def test_it_posts_data_multipart(self):
        with FrescoApp().requestcontext_post(
                files=[('foo', 'foo.txt', 'text/plain', b'bar')]) as c:
            assert c.request.files['foo'].filename == 'foo.txt'
            assert c.request.files['foo'].file.read() == b'bar'

    def test_it_posts_raw_data(self):
        with FrescoApp().requestcontext_post(data=b'xyzzy',
                                             content_type='text/spell') as c:
            assert c.request.body == 'xyzzy'

    def test_it_converts_header_names(self):
        with FrescoApp().requestcontext(user_agent='foo'):
            assert context.request.environ['HTTP_USER_AGENT'] == 'foo'

    def test_it_converts_wsgi_keys(self):
        # CONTENT_TYPE is both a WSGI core environ key and a standard request
        # header. The WSGI key must win.
        with FrescoApp().requestcontext(content_type='foo'):
            assert context.request.environ['CONTENT_TYPE'] == 'foo'

    def test_it_invokes_middleware(self):

        app = FrescoApp()
        app.add_middleware(self._middleware)
        with app.requestcontext() as c:
            assert 'sausages' in c.request.environ

    def test_it_skips_middleware(self):

        app = FrescoApp()
        app.add_middleware(self._middleware)
        with app.requestcontext(middleware=False) as c:
            assert 'sausages' not in c.request.environ

    def test_it_closes_middleware(self):

        close = Mock()

        def middleware(app):
            def middleware(environ, start_response):
                return ClosingIterator(app(environ, start_response), close)
            return middleware

        app = FrescoApp()
        app.add_middleware(middleware)
        with app.requestcontext():
            pass
        assert close.call_count == 1

    def test_it_calls_first_iteration(self):
        """
        Some middleware waits until the first iteration to do things, so make
        sure we trigger this
        """
        from itertools import count
        counter = count()

        def middleware(app):
            def middleware(environ, start_response):
                iterator = app(environ, start_response)
                for item in iterator:
                    yield next(counter)
            return middleware

        app = FrescoApp()
        app.add_middleware(middleware)
        with app.requestcontext():
            assert next(counter) == 1

    def test_it_is_an_error_to_add_middleware_after_the_first_request(self):

        from wsgiref.util import setup_testing_defaults
        app = FrescoApp()
        app.route('/', GET, lambda: Response())
        env = {}
        setup_testing_defaults(env)
        app(env, Mock()).close()
        with pytest.raises(AssertionError):
            app.add_middleware(Mock())


class TestResponseExceptions(object):

    def test_exception_is_converted_to_response(self):

        from fresco.exceptions import RedirectTemporary

        def redirector():
            raise RedirectTemporary('/foo')

        app = FrescoApp()
        app.route('/', GET, redirector)

        with app.requestcontext('/'):
            assert app.view().status_code == 302


class TestUrlfor(object):

    def test_urlfor_on_aliased_functions(self):

        view = lambda: None
        setattr(fixtures, 'aliased_view', view)

        app = FrescoApp()
        app.route('/', GET, view)
        with app.requestcontext():
            assert urlfor(view) == 'http://localhost/'
            assert urlfor('fresco.tests.fixtures.aliased_view') == \
                    'http://localhost/'

        delattr(fixtures, 'aliased_view')

    def test_urlfor_with_view_function(self):

        def view():
            return Response(['ok'])

        app = FrescoApp()
        app.route('/foo', GET, view)
        with app.requestcontext():
            assert urlfor(view) == 'http://localhost/foo'

    def test_urlfor_allows_script_name(self):

        def view():
            return Response(['ok'])

        app = FrescoApp()
        app.route('/foo', GET, view)
        with app.requestcontext():
            assert urlfor(view, _script_name='/abc') ==\
                    'http://localhost/abc/foo'

    def test_urlfor_with_string(self):
        app = FrescoApp()
        app.route('/myviewfunc', GET, fixtures.module_level_function)
        with app.requestcontext():
            assert urlfor('fresco.tests.fixtures.module_level_function') ==\
                    'http://localhost/myviewfunc'

    def test_urlfor_drops_query(self):
        myviewfunc = lambda req: Response([])
        app = FrescoApp()
        app.route('/', GET, myviewfunc)
        with app.requestcontext():
            assert urlfor(myviewfunc) == 'http://localhost/'

    def test_urlfor_generates_first_route(self):

        myviewfunc = lambda req: Response([])
        app = FrescoApp()
        app.route('/1', GET, myviewfunc)
        app.route('/2', GET, myviewfunc)
        with app.requestcontext():
            assert urlfor(myviewfunc) == 'http://localhost/1'

    def test_urlfor_with_class_based_view_spec(self):

        app = FrescoApp()
        app.include('/foo', fixtures.CBV('x'))
        with app.requestcontext():
            assert urlfor('fresco.tests.fixtures.CBV.index_html') == \
                        'http://localhost/foo/'

    def test_it_uses_values_from_path_defaults(self):
        app = FrescoApp()
        app.route('/<test:int>', GET, lambda: None,
                  name='test', test_default=1)
        with app.requestcontext():
            assert urlfor('test') == 'http://localhost/1'

    def test_it_uses_callable_values_from_path_defaults(self):
        generate_value = Mock(return_value=1)
        app = FrescoApp()
        app.route('/<test:int>', GET, lambda: None,
                  name='test', test_default=generate_value)
        with app.requestcontext() as c:
            assert urlfor('test') == 'http://localhost/1'
            generate_value.assert_called_once_with(c.request)

    def test_path_defaults_removed_from_view_kwargs(self):
        app = FrescoApp()
        view = Mock()
        app.route('/<test:int>', GET, view,
                  name='test', test_default=1)
        with app.requestcontext('/2'):
            app.view()
            view.assert_called_with(test=2)



# encoding=UTF-8
from fresco import Route, GET, Response
from fresco.compat import PY3

# A unicode path that can't be represented using ASCII encoding
if PY3:
    unquoted_unicode_path = '/ø'
else:
    unquoted_unicode_path = unicode('/ø', 'UTF-8')

# IRI quoted version of the same path. This is the string the server
# would receive as the HTTP Request-URI
quoted_unicode_path = '/%C3%B8'

# WSGI encoded version of the path. This is the string that appears in the
# environ dict, and will be a native (unicode) string in python3 and a byte
# string in python 2
if PY3:
    wsgi_unicode_path = b'/\xc3\xb8'.decode('latin1')
else:
    wsgi_unicode_path = b'/\xc3\xb8'

# Malformed path, as sent to the server by a non conforming client.
# The code point has been incorrectly encoded in latin-1 instead of
# UTF-8
if PY3:
    misquoted_wsgi_unicode_path = b'/\xf8'.decode('latin1')
else:
    misquoted_wsgi_unicode_path = b'/\xf8'


class CBV(object):
    """
    A class based view
    """

    __routes__ = [
        Route('/', GET, 'index_html'),
        Route('/page', GET, 'view_page'),
        Route('/page2', GET, 'view_page', tag='page2'),
    ]

    def __init__(self, s):
        self.s = s

    def index_html(self):
        return Response([self.s])

    def view_page(self):
        return Response([])


def module_level_function():
    """
    A module level function
    """



#!/usr/bin/env python
# Copyright (c) 2007-2011 Oliver Cope. All rights reserved.
# See LICENSE.txt for terms of redistribution and use.

import os
import re
from setuptools import setup, find_packages

VERSIONFILE = "fresco/__init__.py"


def get_version():
    with open(VERSIONFILE, 'rb') as f:
        return re.search("^__version__\s*=\s*['\"]([^'\"]*)['\"]",
                           f.read().decode('UTF-8'), re.M).group(1)


def read(*path):
    """
    Return content from ``path`` as a string
    """
    with open(os.path.join(os.path.dirname(__file__), *path), 'rb') as f:
        return f.read().decode('UTF-8')

setup(
    name='fresco',
    version=get_version(),
    description='A web micro-framework',
    long_description=read('README.rst') + "\n\n" + read("CHANGELOG.rst"),
    author='Oliver Cope',
    license='BSD',
    author_email='oliver@redgecko.org',
    url='http://www.ollycope.com/software/fresco',
    zip_safe=False,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
    install_requires=['blinker']
)



from collections import defaultdict
from fresco.compat import get_ident

__all__ = 'context', 'RequestContext'


class RequestContext(object):
    """\
    A local storage class that maintains different values for each request
    context in which it is used.

    Requires an ``ident_func`` that returns a hashable identifier that will
    give a different value per request. In a threaded environment this would be
    ``thread.get_ident``. If using a different execution model, a different
    strategy would need to be found.
    """

    def __init__(self, ident_func=get_ident):
        object.__setattr__(self, '_contexts', defaultdict(list))
        object.__setattr__(self, '_ident_func', ident_func)
        self.push()

    def push(self, **bindnames):
        self._contexts[self._ident_func()].append(bindnames)

    def pop(self):
        ident = self._ident_func()
        self._contexts[ident].pop()
        if not self._contexts[ident]:
            del self._contexts[ident]

    def currentcontext(self):
        return self._contexts[self._ident_func()][-1]

    def __getattr__(self, item):
        try:
            return self.currentcontext()[item]
        except (KeyError, IndexError):
            raise AttributeError(item)

    def __setattr__(self, item, ob):
        self.currentcontext()[item] = ob

    def __delattr__(self, item):
        try:
            del self.currentcontext()[item]
        except KeyError:
            raise AttributeError(item)

    # Emulate dictionary access methods
    __getitem__ = __getattr__
    __setitem__ = __setattr__
    __delitem__ = __delattr__

    def get(self, item, default=None):
        return self.currentcontext().get(item, default)

    def __repr__(self):
        return "<%s localdepth=%d; total=%d; current=%r>" % (
                            self.__class__.__name__,
                            len(self._contexts[self._ident_func()]),
                            sum(len(ctx) for ctx in self._contexts.values()),
                            self.currentcontext())


#: The Context for the current request; allows apps to access the current
#: request context as a pseudo-global var
context = RequestContext()



from .util.wsgi import ClosingIterator


__all__ = 'context_middleware', 'XForwarded'


def context_middleware(app, frescoapp):

    from fresco import context

    def context_middleware(environ, start_response):
        context.push(request=frescoapp.request_class(environ), app=frescoapp)
        return ClosingIterator(app(environ, start_response), context.pop)

    return context_middleware


class XForwarded(object):
    """\
    Modify the WSGI environment so that the X_FORWARDED_* headers are observed
    and generated URIs are correct in a proxied environment.

    Use this whenever the WSGI application server is sitting behind
    Apache or another proxy server.

    It is easy for clients to spoof the X-Forwarded-For header. You can largely
    protect against this by listing all trusted proxy server addresses in
    ``trusted``. See http://en.wikipedia.org/wiki/X-Forwarded-For for more
    details.

    HTTP_X_FORWARDED_FOR is substituted for REMOTE_ADDR and
    HTTP_X_FORWARDED_HOST for SERVER_NAME. If HTTP_X_FORWARDED_SSL is set, then
    the wsgi.url_scheme is modified to ``https`` and ``HTTPS`` is set to
    ``on``.

    Example::

        >>> from fresco import FrescoApp, context, GET, Response
        >>> from flea import Agent
        >>> app = FrescoApp()
        >>> @app.route('/', GET)
        ... def view():
        ...     return Response(["URL is ", context.request.url,
        ...                      "; REMOTE_ADDR is ",
        ...                      context.request.remote_addr])
        ...
        >>> r = Agent(XForwarded(app))
        >>> response = r.get('/',
        ...     SERVER_NAME='wsgiserver-name',
        ...     SERVER_PORT='8080',
        ...     HTTP_HOST='wsgiserver-name:8080',
        ...     REMOTE_ADDR='127.0.0.1',
        ...     HTTP_X_FORWARDED_HOST='real-name:81',
        ...     HTTP_X_FORWARDED_FOR='1.2.3.4'
        ... )
        >>> response.body
        u'URL is http://real-name:81/; REMOTE_ADDR is 1.2.3.4'
        >>> response = r.get('/',
        ...     SERVER_NAME='wsgiserver-name',
        ...     SERVER_PORT='8080',
        ...     HTTP_HOST='wsgiserver-name:8080',
        ...     REMOTE_ADDR='127.0.0.1',
        ...     HTTP_X_FORWARDED_HOST='real-name:443',
        ...     HTTP_X_FORWARDED_FOR='1.2.3.4',
        ...     HTTP_X_FORWARDED_SSL='on'
        ... )
        >>> response.body
        u'URL is https://real-name/; REMOTE_ADDR is 1.2.3.4'
    """

    def __init__(self, app, trusted=None):
        self.app = app
        if trusted:
            self.trusted = set(trusted)
        else:
            self.trusted = set()

    def __call__(self, environ, start_response):
        """\
        Call the WSGI app, passing it a modified environ
        """
        is_ssl = environ.get('HTTP_X_FORWARDED_SSL') == 'on'

        if 'HTTP_X_FORWARDED_HOST' in environ:
            host = environ['HTTP_X_FORWARDED_HOST']

            if ':' in host:
                port = host.split(':')[1]
            else:
                port = is_ssl and '443' or '80'

            environ['HTTP_HOST'] = host
            environ['SERVER_PORT'] = port

        if is_ssl:
            environ['wsgi.url_scheme'] = 'https'
            environ['HTTPS'] = 'on'

        try:
            forwards = environ['HTTP_X_FORWARDED_FOR'].split(', ')\
                       + [environ.get('REMOTE_ADDR', '')]
        except KeyError:
            # No X-Forwarded-For header?
            return self.app(environ, start_response)

        if self.trusted:
            for ip in forwards[::-1]:
                # Find the first non-trusted ip; this is our remote address
                if ip not in self.trusted:
                    environ['REMOTE_ADDR'] = ip
                    break
        else:
            environ['REMOTE_ADDR'] = forwards[0]

        return self.app(environ, start_response)



import sys

from collections import defaultdict
from contextlib import contextmanager
from functools import partial
from io import BytesIO

import blinker

from fresco.compat import unquote, urlparse
from fresco.request import Request, HEADER_NAMES as REQUEST_HEADER_NAMES
from fresco.response import Response
from fresco.util.http import encode_multipart
from fresco.util.urls import normpath, make_query
from fresco.util.wsgi import unicode_to_environ

from fresco.exceptions import ResponseException
from fresco.requestcontext import context
from fresco.routing import (GET, ExtensiblePattern, RouteCollection,
                            RouteNotFound)
from fresco.options import Options

__all__ = ('FrescoApp', 'urlfor')

#: The core keys expected in the WSGI environ dict, as defined by PEP3333
WSGI_KEYS = set(['REQUEST_METHOD',
                 'SCRIPT_NAME',
                 'PATH_INFO',
                 'QUERY_STRING',
                 'CONTENT_TYPE',
                 'CONTENT_LENGTH',
                 'SERVER_NAME',
                 'SERVER_PORT',
                 'SERVER_PROTOCOL'])


class FrescoApp(RouteCollection):
    """\
    Fresco application class.
    """

    #: The default class to use for URL pattern matching
    pattern_class = ExtensiblePattern

    #: A stdlib logger object, or None
    logger = None

    #: Class to use to instantiate request objects
    request_class = Request

    def __init__(self, views=None, path=None):

        super(FrescoApp, self).__init__()

        from fresco.middleware import context_middleware

        #: A list of (middleware, args, kwargs) tuples
        self._middleware = []

        #: Middleware layers applied after any middleware added through
        #: :meth:`add_middleware`.
        self.core_middleware = [partial(context_middleware, frescoapp=self)]

        #: The WSGI application. This is generated when the first request is
        #: made.
        self._wsgi_app = None

        #: An options dictionary, for arbitrary application variables or
        #: configuration
        self.options = Options()

        self.signal_ns = blinker.Namespace()

        #: Sent when a request matches a route, immediately before the view is
        #: invoked.
        #: Receivers will be passed: app, route, view, request
        self.route_matched = self.signal_ns.signal('route_matched')

        #: Sent after a view function is invoked, before the response
        #: object is output.
        #: Receivers will be passed: app, view, request and response
        self.view_finished = self.signal_ns.signal('view_finished')

        if views:
            if path is None:
                path = '/'
            self.include(path, views)

    def __call__(self, environ, start_response):
        """\
        Call the app as a WSGI application
        """
        if self._wsgi_app is None:
            self._wsgi_app = self._make_wsgi_app()
        return self._wsgi_app(environ, start_response)

    def __str__(self):
        """\
        String representation of the application and its configured routes
        """
        s = '<%s\n' % self.__class__.__name__
        for route in self.__routes__:
            s += '    ' + str(route)
        s += '>'
        return s

    def view(self, routecollection=None, request=None, path=None, method=None):
        try:
            routecollection = routecollection or self
            request = request or context.request
            method = method or request.method
            path = path or request.path_info
            error_response = None

            context.app = self
            environ = request.environ
            environ['fresco.app'] = self

            if path:
                path = normpath(path)
            else:
                path = '/'
        except ResponseException as e:
            return e.response

        for traversal in routecollection.get_routes(path, method, request):
            try:
                route = traversal.route
                environ['wsgiorg.routing_args'] = (traversal.args,
                                                   traversal.kwargs)
                view = route.getview()
                context.view_self = getattr(view, '__self__', None)

                if self.logger:
                    self.logger.info("matched route: %s %r => %r",
                                     request.method, path, view)

                context.route_traversal = traversal
                self.route_matched.send(self, route=route, view=view,
                                        request=request)

                response = route.getdecoratedview()(*traversal.args,
                                                    **traversal.kwargs)
            except ResponseException as e:
                if e.is_final:
                    return e.response
                error_response = error_response or e.response
            else:
                self.view_finished.send(self, view=view, request=request,
                                        response=response)
                return response

        # A route was matched, but an error was returned
        if error_response:
            return error_response

        # Is this a head request?
        if method == 'HEAD':
            response = self.view(routecollection, request, path, GET)
            if 200 <= response.status_code < 300:
                return response.replace(content=[], content_length=0)
            return response

        # Is the URL matched by another HTTP method?
        methods = self.get_methods(routecollection, request, path)
        if methods:
            return Response.method_not_allowed(methods)

        # Is the URL just missing a trailing '/'?
        if path[-1] != '/':
            for _ in self.get_methods(routecollection, request, path + '/'):
                return Response.redirect_permanent(path + '/')

        return Response.not_found()

    def get_methods(self, routecollection, request, path):
        """\
        Return the HTTP methods valid in routes to the given path
        """
        methods = set()
        for route, _, _, _ in routecollection.get_routes(path, None):
            if route.predicate and not route.predicate(request):
                continue
            methods.update(route.methods)
        return methods

    def add_middleware(self, middleware, *args, **kwargs):
        """\
        Add a WSGI middleware layer
        """
        if self._wsgi_app:
            raise AssertionError("Cannot add middleware: "
                                 "application is now receiving requests")
        self._middleware.append((middleware, args, kwargs))

    def _raw_wsgi_app(self, environ, start_response):
        return self.view(self, self.request_class(environ))(environ,
                                                            start_response)

    def _make_wsgi_app(self, app=None):
        app = app or self._raw_wsgi_app
        for middleware, args, kwargs in self._middleware:
            app = middleware(app, *args, **kwargs)
        for middleware in self.core_middleware:
            app = middleware(app)
        return app

    def urlfor(self, viewspec, *args, **kwargs):
        """\
        Return the url for the given view name or function spec

        :param viewspec: a view name, a reference in the form
                         ``'package.module.viewfunction'``, or the view
                         callable itself.
        :param _scheme: the URL scheme to use (eg 'https' or 'http').
        :param _netloc: the network location to use (eg 'localhost:8000').
        :param _script_name: the SCRIPT_NAME path component
        :param _query: any query parameters, as a dict or list of
                        ``(key, value)`` tuples.
        :param _fragment: a URL fragment to append.

        All other arguments or keyword args are fed to the ``pathfor`` method
        of the pattern.
        """
        scheme = kwargs.pop('_scheme', None)
        netloc = kwargs.pop('_netloc', None)
        query = kwargs.pop('_query', {})
        script_name = kwargs.pop('_script_name', None)
        fragment = kwargs.pop('_fragment', None)

        request = context.request
        traversal = context.get('route_traversal')
        if traversal and traversal.collections_traversed:
            collections_traversed = traversal.collections_traversed
        else:
            collections_traversed = [(self, '')]

        exc = None
        for collection, base_path in collections_traversed[::-1]:
            try:
                path = base_path + \
                        collection.pathfor(viewspec, request=request,
                                           *args, **kwargs)
            except RouteNotFound as e:
                exc = e
                continue
            return request.make_url(scheme=scheme, netloc=netloc,
                                    SCRIPT_NAME=script_name,
                                    PATH_INFO=path,
                                    parameters='',
                                    query=query, fragment=fragment)
        raise exc or RouteNotFound(viewspec)

    def view_method_not_found(self, valid_methods):
        """
        Return a ``405 Method Not Allowed`` response.

        Called when a view matched the pattern but no HTTP methods matched.
        """
        return Response.method_not_allowed(valid_methods)

    @contextmanager
    def requestcontext(self, url='/', environ=None,
                       wsgi_input=b'', middleware=True, **kwargs):
        """
        Return the global :class:`fresco.requestcontext.RequestContext`
        instance, populated with a new request object modelling default
        WSGI environ values.

        Synopsis::

            >>> app = FrescoApp()
            >>> with app.requestcontext('http://www.example.com/view') as c:
            ...     print c.request.url
            ...
            http://www.example.com/view

        Note that ``url`` must be properly URL encoded.

        :param url: The URL for the request,
                    eg ``/index.html`` or ``/search?q=foo``.
        :param environ: values to pass into the WSGI environ dict
        :param wsgi_input: The input stream to use in the ``wsgi.input``
                           key of the environ dict
        :param middleware: If ``False`` the middleware stack will not be
                           invoked. Disabling the middleware can speed up
                           the execution considerably, but it will no longer
                           give an accurate simulation of a real HTTP request.
        :param kwargs: additional keyword arguments will be passed into the
                       WSGI request environment
        """
        url = urlparse(url)
        netloc = url.netloc
        user = ''
        if '@' in netloc:
            user, netloc = netloc.split('@', 1)

        if ':' in user:
            user, _ = user.split(':')[0]

        if isinstance(wsgi_input, bytes):
            wsgi_input = BytesIO(wsgi_input)

        env_overrides = environ or {}
        for key, value in kwargs.items():
            key = key.replace('wsgi_', 'wsgi.')
            if '.' not in key:
                # Convert core WSGI keys to upper case
                if key.upper() in WSGI_KEYS:
                    key = key.upper()
                # Convert header names to form HTTP_USER_AGENT
                elif key.lower() in REQUEST_HEADER_NAMES:
                    key = 'HTTP_' + key.upper()
                # value must be a python native str, whatever that means
                if not isinstance(value, str):
                    value = unicode_to_environ(value)
            env_overrides[key] = value

        environ = {
            'REQUEST_METHOD': 'GET',
            'SERVER_NAME': 'localhost',
            'SERVER_PORT': '80',
            'SERVER_PROTOCOL': 'HTTP/1.0',
            'HTTP_HOST': netloc or 'localhost',
            'SCRIPT_NAME': '',
            'PATH_INFO': unicode_to_environ(unquote(url.path)),
            'REMOTE_ADDR': '127.0.0.1',
            'wsgi.version': (1, 0),
            'wsgi.url_scheme': url.scheme or 'http',
            'wsgi.input': wsgi_input,
            'wsgi.errors': sys.stderr,
            'wsgi.multithread': True,
            'wsgi.multiprocess': True,
            'wsgi.run_once': False,
        }
        if url.scheme == 'https':
            environ['HTTPS'] = 'on'
            environ['SERVER_PORT'] = '443'

        if user:
            environ['REMOTE_USER'] = user

        if url.query:
            environ['QUERY_STRING'] = url.query

        environ.update(env_overrides)

        if middleware:
            if getattr(self, '_fakewsgiapp', None) is None:
                def _fakewsgiapp(environ, start_response):
                    start_response('200 OK', [])
                    yield b''
                self._fakewsgiapp = self._make_wsgi_app(_fakewsgiapp)

            # Invoke a fake wsgi app wrapped in all middleware layers. This
            # executes any middleware, which may have side effects which are
            # expected downstream (eg setting environ keys)
            wsgi_iter = self._fakewsgiapp(environ, lambda *a, **kw: None)
            next(wsgi_iter)
            yield context

            # Exhaust the iterator and close up
            list(wsgi_iter)
            getattr(wsgi_iter, 'close', lambda: None)()

        else:
            context.push(request=self.request_class(environ), app=self)
            yield context
            context.pop()

    def requestcontext_with_payload(
            self, url='/', data=None, environ=None, files=None,
            multipart=False, **kwargs):

        if files:
            multipart = True

        if multipart:
            wsgi_input, headers = encode_multipart(data, files)
            kwargs.update(headers)
        elif hasattr(data, 'read'):
            wsgi_input = data.read()
        elif isinstance(data, bytes):
            wsgi_input = data
        else:
            wsgi_input = make_query(data).encode('ascii')

        if 'CONTENT_LENGTH' not in kwargs:
            kwargs['CONTENT_LENGTH'] = str(len(wsgi_input))

        return self.requestcontext(
            url, environ, wsgi_input=wsgi_input, **kwargs)

    def requestcontext_post(self, *args, **kwargs):
        return self.requestcontext_with_payload(
            REQUEST_METHOD='POST', *args, **kwargs)

    def requestcontext_put(self, *args, **kwargs):
        kwargs['REQUEST_METHOD'] = 'PUT'
        return self.requestcontext_with_payload(*args, **kwargs)

    def requestcontext_patch(self, *args, **kwargs):
        kwargs['REQUEST_METHOD'] = 'PATCH'
        return self.requestcontext_with_payload(*args, **kwargs)

    def requestcontext_delete(self, *args, **kwargs):
        kwargs['REQUEST_METHOD'] = 'DELETE'
        return self.requestcontext(*args, **kwargs)


def collect_keys(items):
    """
    [(k1, v1), (k1, v2), (k2, v3)] -> [(k1, [v1, v2]), (k2, [v3])]
    """
    d = defaultdict(list)
    for key, value in items:
        d[key].append(value)
    return d.items()


def urlfor(*args, **kwargs):
    """\
    Convenience wrapper around :meth:`~fresco.core.FrescoApp.urlfor`.
    """
    return context.app.urlfor(*args, **kwargs)



"""
Security-related utilities
"""

def check_equal_constant_time(a, b):
    """
    Return ``True`` if string ``a`` is equal to string ``b``.

    If ``a`` and ``b`` are of the same length, this function will take the same
    amount of time to execute, regardless of whether or not a and b are equal.
    """
    if len(a) != len(b):
        return False
    result = 0
    for c1, c2 in zip(a, b):
        result |= ord(c1) ^ ord(c2)
    return result == 0



# Copyright (c) 2007-2012 Oliver Cope. All rights reserved.
# See LICENSE.txt for terms of redistribution and use.

import copy
try:
    from urllib.parse import quote, unquote
except ImportError:
    from urllib import quote, unquote  # NOQA
from datetime import datetime, timedelta
from calendar import timegm
from email.utils import formatdate


class Cookie(object):
    """
    Represents an HTTP cookie.

    See rfc2109, HTTP State Management Mechanism

    Example:

        >>> from fresco.cookie import Cookie
        >>> c = Cookie('session_id', 'abc123')
        >>> c.path = '/cgi-bin'
        >>> c.domain = '.example.org'
        >>> c.path
        '/cgi-bin'
        >>> str(c)
        'session_id=abc123;Domain=.example.org;Path=/cgi-bin;Version=1'
    """
    attributes = [
        ("Comment", "comment"),
        ("Domain", "domain"),
        ("Expires", "expires"),
        ("Max-Age", "maxage"),
        ("Path", "path"),
        ("Secure", "secure"),
        ("Version", "version"),
    ]
    attribute_dict = dict(attributes)

    def __init__(
        self, name, value, maxage=None, expires=None, path=None,
        secure=None, domain=None, comment=None, http_only=False,
        version=1
    ):
        """
        Initialize a ``Cookie`` instance.
        """
        self.name = name
        self.value = value
        self.maxage = maxage
        self.path = path
        self.secure = secure
        self.domain = domain
        self.comment = comment
        self.version = version
        self.expires = expires
        self.http_only = http_only

    def __str__(self):
        """
        Returns a string representation of the cookie in the format, eg
        ``session_id=abc123;Path=/cgi-bin;Domain=.example.com;Version=1``
        """
        cookie = ['%s=%s' % (self.name, quote(str(self.value)))]
        for cookie_name, att_name in self.attributes:
            value = getattr(self, att_name, None)
            if value is not None:
                cookie.append('%s=%s' % (cookie_name, str(value)))
        if self.http_only:
            cookie.append('HttpOnly')
        return ';'.join(cookie)

    def set_expires(self, dt):
        """
        Set the cookie ``expires`` value to ``datetime`` object ``dt``
        """
        self._expires = dt

    def get_expires(self):
        """
        Return the cookie ``expires`` value as an instance of ``datetime``.
        """
        if self._expires is None and self.maxage is not None:
            if self.maxage == 0:
                # Make sure immediately expiring cookies get a date firmly in
                # the past.
                self._expires = datetime(1980, 1, 1)
            else:
                self._expires = datetime.now() + timedelta(seconds=self.maxage)

        if isinstance(self._expires, datetime):
            return formatdate(timegm(self._expires.utctimetuple()))

        else:
            return self._expires

    expires = property(get_expires, set_expires)


def expire_cookie(cookie_or_name, *args, **kwargs):
    """
    Synopsis:

        >>> from fresco import Response
        >>> from fresco.cookie import expire_cookie
        >>> def view():
        ...     return Response(set_cookie=expire_cookie('X', path='/'))
        ...
        >>> from fresco import FrescoApp
        >>> with FrescoApp().requestcontext() as c:
        ...     print(view().get_header('Set-Cookie'))
        ...
        X=;Expires=Tue, 01 Jan 1980 00:00:00 -0000;Max-Age=0;Path=/;Version=1
    """
    if isinstance(cookie_or_name, Cookie):
        expire = cookie_or_name
    else:
        expire = Cookie(name=cookie_or_name, value='', *args, **kwargs)
    return Cookie(
        name=expire.name,
        value='',
        expires=datetime(1980, 1, 1),
        maxage=0,
        domain=kwargs.get('domain', expire.domain),
        path=kwargs.get('path', expire.path)
    )


def parse_cookie_header(cookie_string, unquote=unquote):
    """
    Return a list of Cookie objects read from the request headers.

    :param cookie_string: The cookie, eg ``CUSTOMER=FRED; path=/;``

    :param unquote:       A function to decode quoted values. If ``None``
                          values will be left unchanged

    See rfc2109, section 4.4

    The cookie header should be a ';' separated list of name value pairs. If a
    name is prefixed by a '$', then that name is an attribute of the most
    recently (left to right) encountered cookie.  If no cookie has yet been
    parsed then the value applies to the cookie mechanism as a whole.
    """

    if unquote is None:
        unquote = lambda v: v

    if not cookie_string:
        return []
    cookies = []

    # Here we put the $ prefixed attributes that appear *before* a
    # named cookie, to use as a template for other cookies.
    cookie_template = Cookie(None, None)

    for part in cookie_string.split(";"):

        if not '=' in part:
            continue

        k, v = part.strip().split("=", 1)

        # Unquote quoted values ('"..."' => '...')
        if v and '"' == v[0] == v[-1] and len(v) > 1:
            v = v[1:-1]

        if k[0] == '$':
            # Value pertains to most recently read cookie,
            # or cookie_template
            k = k[1:]
            if len(cookies) == 0:
                cookie = copy.copy(cookie_template)
            else:
                cookie = cookies[-1]

            try:
                setattr(cookie, cookie.attribute_dict[k], v)
            except KeyError:
                pass
        else:
            cookies.append(copy.copy(cookie_template))
            cookies[-1].name = unquote(k)
            cookies[-1].value = unquote(v)
    return cookies
